@@ -1,5 +1,5 @@
 /*
- * @(#)$Id: acconfig.h,v 1.17 1994/10/16 00:35:31 mrg stable $
+ * @(#)$Id: acconfig.h,v 1.7 2002/01/25 22:12:23 crazyed Exp $
  */
 
 /*
@@ -33,6 +33,9 @@
 /* define this if you have setenv(3) */
 #undef HAVE_SETENV
 
+/* define this if you have unsetenv(3) */
+#undef HAVE_UNSETENV
+
 /* define this if you have setsid(2) */
 #undef HAVE_SETSID
 
@@ -192,6 +195,11 @@
 #undef Rlisten
 #undef Rselect
 
+/*
+ *  Perl support.
+ */
+#undef PERL
+
 /* Define this is DIRSIZ takes no argument */
 #undef DIRSIZ_TAKES_NO_ARG
 
@@ -231,3 +239,10 @@
 /* Define this if you have hstrerror, for h_errno use */
 #undef HAVE_HSTRERROR
 
+/* Define this if you want OPENSSL support */
+#undef HAVE_SSL
+/* Define this if you want OPENSSL2 support */
+#undef WANT_SSL2
+/* Define this if you have sysctlbyname() */
+#undef HAVE_SYSCTLBYNAME
+
