@@ -150,14 +150,6 @@
  */
 #define MIRC_BROKEN_DCC_RESUME
 
-/*
- * Define this if you want the traditional EPIC behavior of checksumming
- * files before they are sent and verification after they are recieved.
- * If this is #undef'd, you will not send out checksums, nor will you use
- * any checksums if they are recieved (from other epic clients).
- */
-#define USE_DCC_CHECKSUM
-
 /*
  * Youll want to define this if your system is missing the glob()
  * call, or if its broken (solaris).
@@ -319,12 +311,14 @@
  * remove them.  They are default values for variables and are required for
  * proper compilation.
  */
+#define DEFAULT_ALLOW_C1_CHARS 0
 #define DEFAULT_ALT_CHARSET 1
 #define DEFAULT_ALWAYS_SPLIT_BIGGEST 1
 #define DEFAULT_AUTO_NEW_NICK 1
 #define DEFAULT_AUTO_RECONNECT 1
 #define DEFAULT_AUTO_RECONNECT_DELAY 0
 #define DEFAULT_AUTO_REJOIN 1
+#define DEFAULT_AUTO_REJOIN_CONNECT 1
 #define DEFAULT_AUTO_REJOIN_DELAY 0
 #define DEFAULT_AUTO_UNMARK_AWAY 0
 #define DEFAULT_AUTO_WHOWAS 1
@@ -394,7 +388,6 @@
 #define DEFAULT_NEW_SERVER_LASTLOG_LEVEL "ALL"
 #define DEFAULT_NO_CTCP_FLOOD 1
 #define DEFAULT_NO_FAIL_DISCONNECT 0
-#define DEFAULT_NOTIFY_HANDLER "NOISY"
 #define DEFAULT_NOTIFY_INTERVAL 60
 #define DEFAULT_NOTIFY_LEVEL "ALL"
 #define DEFAULT_NOTIFY_ON_TERMINATION 1
@@ -421,6 +414,9 @@
 #define DEFAULT_STATUS_AWAY " (Away)"
 #define DEFAULT_STATUS_CHANNEL " %C"
 #define DEFAULT_STATUS_CHANOP "@"
+#define DEFAULT_STATUS_HALFOP "%"
+#define DEFAULT_STATUS_SSL_ON "*SSL*"
+#define DEFAULT_STATUS_SSL_OFF "*RAW*"
 #define DEFAULT_STATUS_CLOCK " %T"
 #define DEFAULT_STATUS_CPU_SAVER " (%L)"
 #define DEFAULT_STATUS_FORMAT "%T [%R] %*%=%@%N%#%S%H%B%Q%A%C%+%I%O%M%F%L %D %U %W"
@@ -565,5 +561,7 @@
 #endif
 /* end of section not to change */
 
+#undef ALLOC_DEBUG
+
 #endif /* _CONFIG_H_ */
 
