@@ -13,7 +13,8 @@
 
 #define IRCII_COMMENT   "Accept no limitations."
 #define IRCRC_NAME 	"/.ircrc"
-#define EMAIL_CONTACT 	"ircii-epic@concentric.net"
+#define EPICRC_NAME 	"/.epicrc"
+#define EMAIL_CONTACT 	"list@epicsol.org"
 
 /*
  * Here you can set the in-line quote character, normally backslash, to
@@ -114,6 +115,8 @@ extern		char	hostname[];
 extern const 	char 	internal_version[];
 extern		char *	invite_channel;
 extern		char *	ircrc_file;
+extern		char *	epicrc_file;
+extern		char *	startup_file;
 extern		char *	irc_lib;
 extern		char *	irc_path;
 extern const 	char 	irc_version[];
@@ -129,6 +132,7 @@ extern		char	one[];
 extern		char *	public_nick;
 extern		char	realname[];
 extern	const	char	ridiculous_version_name[];
+extern	const unsigned long commit_id;
 extern		char *	send_umode;
 extern	const	char *	unknown_userhost;
 extern	const	char 	useful_info[];
@@ -140,31 +144,20 @@ extern		char	comma[];
 
 extern struct	in_addr	LocalHostAddr;
 extern		fd_set  readables;
-extern		time_t	start_time;
-extern		time_t	idle_time;
-extern		time_t	now;
+extern struct timeval 	start_time;
+extern struct timeval	idle_time;
+extern struct timeval	now;
 
 /* irc.c's extern functions */
 	char	get_a_char 		(void);
 	void	get_line_return 	(char, char *);
 	void	get_line 		(char *, int, void (*) (char, char *));
 	void	io 			(const char *);
-	void	irc_exit 		(int, char *, ...) __A(2) __N;
+	void	irc_exit 		(int, char *, ...) /*__A(2)*/ __N;
 	void	irc_quit 		(char, char *);
-	int	wild_match 		(const char *, const char *); 
-							 /* XXX reg.c */
+#include "reg.h"
 	void	dump_load_stack		(int);	 /* XXX command.c */
 const	char *	current_filename	(void);  /* XXX command.c */
 	int	current_line		(void);	 /* XXX command.c */
 	char *	current_package		(void);  /* XXX command.c */
-
-#if 0
-	char *	bsd_getenv		(const char *);	/* XXX compat.c */
-	int	bsd_putenv		(const char *);	/* XXX compat.c */
-	int	bsd_setenv		(const char *, const char *, int); 
-						/* XXX compat.c */
-	void	bsd_unsetenv		(const char *);	/* XXX compat.c */
-#endif
-
-
 #endif /* __irc_h */
