@@ -7,7 +7,7 @@
  *
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
  *
- * @(#)$Id: ircaux.h,v 1.2 2001/01/20 00:36:30 jnelson Exp $
+ * @(#)$Id: ircaux.h,v 1.12 2002/02/14 16:33:25 jnelson Exp $
  */
 
 #ifndef _IRCAUX_H_
@@ -20,24 +20,30 @@ typedef int 	comp_func 		(char *, char *);
 
 #define new_malloc(x) really_new_malloc	((x), __FILE__, __LINE__)
 #define new_free(x)   really_new_free	((void **)(x), __FILE__, __LINE__)
+#define new_realloc(x,y) really_new_realloc ((x), (y), __FILE__, __LINE__)
 #define MUST_BE_MALLOCED(x, y) \
 		fatal_malloc_check ((void *)(x), (y), __FILE__, __LINE__)
 #define RESIZE(x, y, z) new_realloc	((void **)& (x), sizeof(y) * (z))
 #define LOCAL_COPY(y) strcpy(alloca(strlen((y)) + 1), y)
+#define m_e3cat(x,y,z) m_ec3cat((x),(y),(z),NULL)
+#define m_s3cat(x,y,z) m_sc3cat((x),(y),(z),NULL)
+#define m_s3cat_s(x,y,z) m_sc3cat_s((x),(y),(z),NULL)
+#define m_3cat(x,y,z) m_c3cat((x),(y),(z),NULL)
 
 extern	int	need_delayed_free;
 void	fatal_malloc_check	(void *, const char *, char *, int);
 void *	really_new_malloc 	(size_t, char *, int);
 void *	really_new_free 	(void **, char *, int);
 int	debug_new_free		(void **, char *, int);
-void *	new_realloc 		(void **, size_t);
+void *	really_new_realloc 	(void **, size_t, char *, int);
+void	malloc_dump		(char *);
 
 char *	check_nickname 		(char *, int);
 char *	next_arg 		(char *, char **);
 char *	new_next_arg 		(char *, char **);
 char *	new_new_next_arg 	(char *, char **, char *);
 char *	s_next_arg		(char **);
-char *	last_arg 		(char **);
+char *	last_arg 		(char **, size_t *cluep);
 char *	expand_twiddle 		(char *);
 char *	upper 			(char *);
 char *	lower 			(char *);
@@ -48,10 +54,10 @@ char *	double_quote 		(const char *, const char *, char *);
 char *	malloc_strcpy 		(char **, const char *);
 char *	malloc_strcat 		(char **, const char *);
 char *	malloc_str2cpy		(char **, const char *, const char *);
-char *	m_s3cat_s 		(char **, const char *, const char *);
-char *	m_s3cat 		(char **, const char *, const char *);
-char *	m_3cat 			(char **, const char *, const char *);
-char *	m_e3cat 		(char **, const char *, const char *);
+char *	m_sc3cat_s 		(char **, const char *, const char *, size_t *clue);
+char *	m_sc3cat 		(char **, const char *, const char *, size_t *clue);
+char *	m_c3cat			(char **, const char *, const char *, size_t *clue);
+char *	m_ec3cat 		(char **, const char *, const char *, size_t *clue);
 char *	m_2dup 			(const char *, const char *);
 char *	m_3dup 			(const char *, const char *, const char *);
 char *	m_opendup 		(const char *, ...) __A(1);
@@ -75,8 +81,10 @@ char *	m_strcat_ues 		(char **, char *, int);
 char *	strmopencat		(char *, int, ...);
 char *	stristr 		(const char *, const char *);
 char *	rstristr 		(const char *, const char *);
+char *	findchar		(char *, int);
 FILE *	uzfopen 		(char **, char *, int);
 int	end_strcmp 		(const char *, const char *, int);
+char*   exec_pipe		(char *, char *, size_t *, char**);
 void	panic 			(char *, ...) __A(1) __N;
 int	vt100_decode 		(char);
 int	count_ansi		(char *, int);
@@ -85,13 +93,18 @@ int	lw_strcmp 		(comp_func *, char *, char *);
 int	open_to 		(char *, int, int);
 struct timeval get_time 	(struct timeval *);
 double 	time_diff 		(struct timeval, struct timeval);
+struct timeval time_add		(struct timeval, struct timeval);
+struct timeval time_subtract	(struct timeval, struct timeval);
+struct timeval double_to_timeval (double);
 char *	plural 			(int);
-int	time_to_next_minute 	(void);
-char *	remove_trailing_spaces 	(char *);
+struct timeval	time_to_next_minute 	(void);
+char *	remove_trailing_spaces 	(char *, size_t *cluep);
 char *	ltoa 			(long);
 char *	ftoa			(double);
 char *	strformat 		(char *, const char *, int, int);
 char *	chop_word 		(char *);
+char *	skip_spaces		(char *);
+int	split_args		(char *, char **to, size_t);
 int	splitw 			(char *, char ***);
 char *	unsplitw 		(char ***, int);
 int	check_val 		(char *);
@@ -108,7 +121,8 @@ long	my_atol 		(const char *);
 u_long	hashpjw 		(char *, u_long);
 char *	m_dupchar 		(int);
 char *	strmccat		(char *, char, int);
-off_t	file_size		(char *);
+off_t	file_size		(const char *);
+int	file_exists		(const char *);
 int	is_root			(const char *, const char *, int);
 size_t	streq			(const char *, const char *);
 char *	m_strndup		(const char *, size_t);
@@ -132,7 +146,8 @@ char *	encode			(const char *, int);
 char *	decode			(const char *);
 char *	chomp			(char *);
 int 	opento			(const char *, int, off_t);
-int	figure_out_address	(char *, char **, char **, char **, char **, int *);
+int	figure_out_address	(char *, char **, char **, char **);
+int	figure_out_domain	(char *, char **, char **, int *);
 int	count_char		(const unsigned char *, const unsigned char);
 char *	strnrchr		(char *, char, int);
 void	mask_digits		(char **);
@@ -141,7 +156,7 @@ char *  strmpcat		(char *, size_t, const char *, ...) __A(3);
 u_char *strcpy_nocolorcodes	(u_char *, const u_char *);
 u_long	random_number		(u_long);
 char *	urlencode		(const char *);
-char *	urldecode		(char *);
+char *	urldecode		(char *, size_t *);
 const char *	find_forward_quote	(const char *, const char *);
 const char *	find_backward_quote	(const char *, const char *);
 const char *	my_strerror		(int);
