@@ -25,6 +25,7 @@ struct WindowStru;
 	int	is_on_channel		(Char *, Char *);
 	int	is_chanop		(Char *, Char *);
 	int	is_chanvoice		(Char *, Char *);
+	int	is_halfop		(Char *, Char *);
 	int	number_on_channel	(Char *, int);
 	char *	create_nick_list	(Char *, int);
 	char *	create_chops_list	(Char *, int);
@@ -37,6 +38,7 @@ struct WindowStru;
 	void	list_channels		(void);
 	void	switch_channels		(char, char *);
 	void	change_server_channels	(int, int);
+	void	destroy_waiting_channels	(int);
 	void	destroy_server_channels	(int);
 	void	reconnect_all_channels	(void);
 	Char *	what_channel		(Char *);
@@ -44,6 +46,7 @@ struct WindowStru;
 	Char *	fetch_userhost		(int, Char *);
 	int	get_channel_oper	(Char *, int);
 	int	get_channel_voice	(Char *, int);
+	int     get_channel_halfop	(Char *, int);
 struct WindowStru *get_channel_window	(Char *, int);
 	void	set_channel_window	(struct WindowStru *, Char *);
 	char *	create_channel_list	(int);
