@@ -17,6 +17,10 @@
 /* To get definition of Notify */
 #include "notify.h"
 
+#ifdef HAVE_SSL
+#include "ssl.h"
+#endif
+
 /* XXXX Ick.  Gross.  Bad. XXX */
 struct notify_stru;
 
@@ -27,6 +31,7 @@ typedef	struct
 	char	*itsname;		/* the server's idea of its name */
 	char	*password;		/* password for that server */
 	int	port;			/* port number on that server */
+	char	*group;			/* Server group it belongs to */
 	char	*nickname;		/* Authoritative nickname for us */
 	char	*s_nickname;		/* last NICK command sent */
 	char	*d_nickname;		/* Default nickname to use */
@@ -69,6 +74,12 @@ typedef	struct
 	int	closing;		/* True if close_server called */
 	int	reconnect_to;		/* Server to connect to on EOF */
 	char	*quit_message;		/* Where we stash a quit message */
+#ifdef HAVE_SSL
+	SSL_CTX*	ctx;
+	SSL_METHOD*	meth;
+	int	enable_ssl;
+	SSL*	ssl_fd;
+#endif
 }	Server;
 extern	Server	*server_list;
 #endif	/* NEED_SERVER_LIST */
@@ -100,22 +111,23 @@ extern	int	parsing_server_index;
 
 	void	add_to_server_list 		(const char *, int, 
 						 const char *, const char *, 
-						 int);
+						 const char *, int);
 	int	find_in_server_list		(const char *, int);
 	int	find_server_refnum		(char *, char **rest);
 	int	parse_server_index		(const char *);
 	void	parse_server_info		(char *, char **, char **,
-						 char **);
-	void	build_server_list		(char *);
+						 char **, char **);
+	void	build_server_list		(char *, char *);
 	int	read_server_file		(void);
 	void	display_server_list		(void);
 	char *	create_server_list		(void);	/* MALLOC */
 	int	server_list_size		(void);
 
 	void	do_server 			(fd_set *);
-	void	flush_server			(void);
+	void	flush_server			(int);
 	void	send_to_aserver			(int, const char *, ...) __A(2);
 	void	send_to_server			(const char *, ...) __A(1);
+	void	send_to_server_raw		(size_t len, const char *buffer);
 	void	clear_sent_to_server		(int);
 	int	sent_to_server			(int);
 
@@ -123,6 +135,7 @@ extern	int	parsing_server_index;
 	int	close_all_servers		(const char *);
 	void	close_server			(int, const char *);
 	void	save_server_channels		(int);
+	void	dont_save_server_channels	(int);
 
 	void	set_server_away			(int, const char *);
 const	char *	get_server_away			(int);
@@ -138,10 +151,13 @@ const	char *	get_umode			(int);
 
 const	char *	get_server_name			(int);
 const	char *	get_server_itsname		(int);
+const	char *	get_server_group		(int);
 	void	set_server_itsname		(int, const char *);
 	void	set_server_version_string	(int, const char *);
 const 	char *	get_server_version_string	(int);
-
+	int	get_server_isssl		(int);
+const	char *	get_server_cipher		(int);
+ 
 	void	register_server			(int, const char *);
 	void	server_registration_is_not_pending (int);
 	void	password_sendline		(char *, char *);
@@ -151,7 +167,7 @@ const 	char *	get_server_version_string	(int);
 	void	server_is_connected		(int, int);
 	int	auto_reconnect_callback		(void *);
 	int	server_reconnects_to		(int, int);
-	int	reconnect			(int);
+	int	reconnect			(int, int);
 
 	int	get_server_port			(int);
 	int	get_server_local_port		(int);
@@ -191,5 +207,7 @@ const	char *	get_server_redirect		(int);
 	int     set_server_quit_message 	(int, const char *message);
 const char *    get_server_quit_message		(int);
 
+	void	clear_reconnect_counts		(void);
 
+	void   	set_server_enable_ssl 		(int, int);
 #endif /* _SERVER_H_ */
