@@ -39,23 +39,18 @@
 #include "alias.h"
 #include "alist.h"
 #include "array.h"
-#include "dcc.h"
 #include "commands.h"
 #include "files.h"
 #include "history.h"
 #include "hook.h"
 #include "input.h"
 #include "ircaux.h"
-#include "names.h"
 #include "output.h"
-#include "parse.h"
 #include "screen.h"
-#include "server.h"
 #include "stack.h"
 #include "status.h"
 #include "vars.h"
 #include "window.h"
-#include "notify.h"
 #include "keys.h"
 
 #define LEFT_BRACE '{'
@@ -278,13 +273,13 @@ extern u_32int_t       char_searches;
 	    say("Total bytes allocated to aliases: %ld", 
 			alias_total_bytes_allocated);
 
-	    say("Var command hits/misses/passes/missed-by [%ld/%ld/%ld/%3.1f]",
+	    say("Var cache hits/misses/passes/missed-by [%ld/%ld/%ld/%3.1f]",
 			var_cache_hits, 
 			var_cache_misses, 
 			var_cache_passes, 
 			( var_cache_misses ? (double) 
 			  (var_cache_missed_by / var_cache_misses) : 0.0));
-	    say("Cmd command hits/misses/passes/missed-by [%ld/%ld/%ld/%3.1f]",
+	    say("Cmd cache hits/misses/passes/missed-by [%ld/%ld/%ld/%3.1f]",
 			cmd_cache_hits, 
 			cmd_cache_misses, 
 			cmd_cache_passes,
@@ -569,6 +564,12 @@ BUILT_IN_COMMAND(localcmd)
 	if (!args)
 		args = empty_string;
 
+	if (!my_strnicmp(name, "-dump", 2))	/* Illegal name anyways */
+	{
+		destroy_aliases(VAR_ALIAS_LOCAL);
+		return;
+	}
+
 	while (name && *name)
 	{
 		char 	*next_name;
@@ -1623,8 +1624,7 @@ void	list_cmd_alias (char *name)
 		if (!name || !strncmp(cmd_alias.list[cnt]->name, name, len))
 		{
 			script = cmd_alias.list[cnt]->filename[0]
-				? m_strdup(cmd_alias.list[cnt]->filename)
-				: "*";
+				? cmd_alias.list[cnt]->filename : "*";
 
 			if ((s = strchr(cmd_alias.list[cnt]->name + len, '.')))
 			{
@@ -1756,36 +1756,39 @@ char *	get_cmd_alias (char *name, int *howmany, char **complete_name, void **arg
 
 /*
  * This function is strictly O(N).  This should probably be addressed.
+ *
+ * Updated as per get_subarray_elements.
  */
 char **	glob_cmd_alias (char *name, int *howmany)
 {
-	int	cnt;
-	int	cmp;
-	int 	len;
-	char 	**matches = NULL;
-	int 	matches_size = 5;
+	int	pos, max;
+	int    	cnt;
+	int     len;
+	char    **matches = NULL;
+	int     matches_size = 0;
 
 	len = strlen(name);
 	*howmany = 0;
-	matches = RESIZE(matches, char *, matches_size);
+	if (len) {
+		find_array_item((array*)&cmd_alias, name, &max, &pos);
+	} else {
+		pos = 0;
+		max = cmd_alias.max;
+	}
+	if (0 > max) max = -max;
 
-	for (cnt = 0; cnt < cmd_alias.max; cnt++)
+	for (cnt = 0; cnt < max; cnt++, pos++)
 	{
-		if (!(cmp = strncmp(name, cmd_alias.list[cnt]->name, len)))
-		{
-			if (strchr(cmd_alias.list[cnt]->name + len, '.'))
-				continue;
+		if (strchr(cmd_alias.list[pos]->name + len, '.'))
+			continue;
 
-			matches[*howmany] = m_strdup(cmd_alias.list[cnt]->name);
-			*howmany += 1;
-			if (*howmany == matches_size)
-			{
-				matches_size += 5;
-				RESIZE(matches, char *, matches_size);
-			}
+		if (*howmany >= matches_size)
+		{
+			matches_size += 5;
+			RESIZE(matches, char *, matches_size + 1);
 		}
-		else if (cmp < 0)
-			break;
+		matches[*howmany] = m_strdup(cmd_alias.list[pos]->name);
+		*howmany += 1;
 	}
 
 	if (*howmany)
@@ -1798,36 +1801,39 @@ char **	glob_cmd_alias (char *name, int *howmany)
 
 /*
  * This function is strictly O(N).  This should probably be addressed.
+ *
+ * Updated as per get_subarray_elements.
  */
 char **	glob_assign_alias (char *name, int *howmany)
 {
+	int	pos, max;
 	int    	cnt;
-	int     cmp;
 	int     len;
 	char    **matches = NULL;
-	int     matches_size = 5;
+	int     matches_size = 0;
 
 	len = strlen(name);
 	*howmany = 0;
-	matches = RESIZE(matches, char *, matches_size);
+	if (len) {
+		find_array_item((array*)&var_alias, name, &max, &pos);
+	} else {
+		pos = 0;
+		max = var_alias.max;
+	}
+	if (0 > max) max = -max;
 
-	for (cnt = 0; cnt < var_alias.max; cnt++)
+	for (cnt = 0; cnt < max; cnt++, pos++)
 	{
-		if (!(cmp = strncmp(name, var_alias.list[cnt]->name, len)))
-		{
-			if (strchr(var_alias.list[cnt]->name + len, '.'))
-				continue;
+		if (strchr(var_alias.list[pos]->name + len, '.'))
+			continue;
 
-			matches[*howmany] = m_strdup(var_alias.list[cnt]->name);
-			*howmany += 1;
-			if (*howmany == matches_size)
-			{
-				matches_size += 5;
-				RESIZE(matches, char *, matches_size);
-			}
+		if (*howmany >= matches_size)
+		{
+			matches_size += 5;
+			RESIZE(matches, char *, matches_size + 1);
 		}
-		else if (cmp < 0)
-			break;
+		matches[*howmany] = m_strdup(var_alias.list[pos]->name);
+		*howmany += 1;
 	}
 
 	if (*howmany)
@@ -1910,19 +1916,26 @@ char **	pmatch_assign_alias (char *name, int *howmany)
 	return matches;
 }
 
-
 /*
  * This function is strictly O(N).  This should probably be addressed.
+ *
+ * OK, so it isn't _strictly_ O(N) anymore, however, it is still O(N)
+ * for N being all elements below the subarray being requested.  This
+ * makes recursive /foreach's such as purge run much faster, however,
+ * since each variable will be tested no more than its current depth
+ * times, rather than every single time a /foreach is performed.
+ *
+ * In the worst case scenario where the entire variable space consists
+ * of a single flat subarray, the new algorithm would perform no worse
+ * than the old.
  */
 char **	get_subarray_elements (char *root, int *howmany, int type)
 {
 	AliasSet *as;		/* XXXX */
-
-	int len, len2;
-	int cnt;
+	int pos, cnt, max;
 	int cmp = 0;
 	char **matches = NULL;
-	int matches_size = 5;
+	int matches_size = 0;
 	size_t end;
 	char *last = NULL;
 
@@ -1930,32 +1943,27 @@ char **	get_subarray_elements (char *root, int *howmany, int type)
 		as = &cmd_alias;
 	else
 		as = &var_alias;
+	root = m_2dup(root, ".");
+	find_array_item((array*)as, root, &max, &pos);
 
-	len = strlen(root);
+	if (0 > max) max = -max;
 	*howmany = 0;
-	matches = RESIZE(matches, char *, matches_size);
+	cmp = strlen(root);
+	new_free(&root);
 
-	for (cnt = 0; cnt < as->max; cnt++)
+	for (cnt = 0; cnt < max; cnt++, pos++)
 	{
-		len2 = strlen(as->list[cnt]->name);
-		if ( (len < len2) &&
-		     ((cmp = streq(root, as->list[cnt]->name)) == len))
+		end = strcspn(ARRAY_ITEM(as, pos)->name + cmp, ".");
+		if (last && !my_strnicmp(ARRAY_ITEM(as, pos)->name, last, cmp + end))
+			continue;
+		if (*howmany >= matches_size)
 		{
-			if (as->list[cnt]->name[cmp] == '.')
-			{
-				end = strcspn(as->list[cnt]->name + cmp + 1, ".");
-				if (last && !my_strnicmp(as->list[cnt]->name, last, cmp + 1 + end))
-					continue;
-				matches[*howmany] = m_strndup(as->list[cnt]->name, cmp + 1 + end);
-				last = matches[*howmany];
-				*howmany += 1;
-				if (*howmany == matches_size)
-				{
-					matches_size += 5;
-					RESIZE(matches, char *, matches_size);
-				}
-			}
+			matches_size = *howmany + 5;
+			RESIZE(matches, char *, matches_size + 1);
 		}
+		matches[*howmany] = m_strndup(ARRAY_ITEM(as, pos)->name, cmp + end);
+		last = matches[*howmany];
+		*howmany += 1;
 	}
 
 	if (*howmany)
@@ -2353,6 +2361,7 @@ char 	*aliasctl 	(char *input)
 		{
 			char **mlist = NULL;
 			char *mylist = NULL;
+			size_t	mylistclue = 0;
 			int num = 0, ctr;
 
 			if (!my_stricmp(listc, "*"))
@@ -2367,7 +2376,7 @@ char 	*aliasctl 	(char *input)
 
 			for (ctr = 0; ctr < num; ctr++)
 			{
-				m_s3cat(&mylist, space, mlist[ctr]);
+				m_sc3cat(&mylist, space, mlist[ctr], &mylistclue);
 				new_free((char **)&mlist[ctr]);
 			}
 			new_free((char **)&mlist);
@@ -2379,6 +2388,7 @@ char 	*aliasctl 	(char *input)
 		{
 			char **	mlist = NULL;
 			char *	mylist = NULL;
+			size_t	mylistclue = 0;
 			int	num = 0,
 				ctr;
 
@@ -2389,7 +2399,7 @@ char 	*aliasctl 	(char *input)
 
 			for (ctr = 0; ctr < num; ctr++)
 			{
-				m_s3cat(&mylist, space, mlist[ctr]);
+				m_sc3cat(&mylist, space, mlist[ctr], &mylistclue);
 				new_free((char **)&mlist[ctr]);
 			}
 			new_free((char **)&mlist);
