@@ -21,6 +21,7 @@
 #include "dcc.h"
 #include "commands.h"
 #include "exec.h"
+#include "files.h"
 #include "funny.h"
 #include "help.h"
 #include "history.h"
@@ -165,6 +166,7 @@ static 	void	waitcmd 	(const char *, char *, const char *);
 static	void	whois 		(const char *, char *, const char *);
 static	void	xevalcmd 	(const char *, char *, const char *);
 static	void	xtypecmd 	(const char *, char *, const char *);
+static	void	allocdumpcmd	(const char *, char *, const char *);
 
 /* other */
 static	void	eval_inputlist 	(char *, char *);
@@ -196,6 +198,7 @@ static	IrcCommand irc_command[] =
         { "ABORT",      NULL,           abortcmd,               0 },
 	{ "ADMIN",	"ADMIN",	send_comm, 		0 },
 	{ "ALIAS",	zero,		aliascmd,		0 }, /* alias.c */
+	{ "ALLOCDUMP",	NULL,		allocdumpcmd,		0 },
 	{ "ASSIGN",	one,		assigncmd,		0 }, /* alias.c */
 	{ "AWAY",	"AWAY",		away,			0 },
 	{ "BEEP",	0,		beepcmd,		0 },
@@ -633,26 +636,7 @@ BUILT_IN_COMMAND(e_channel)
 {
 	if (args && *args)
 	{
-		char *blah = (char *)alloca(strlen(args) * 2 + 4);
-
-		/* XXX Oh well. */
-		if (args[0] == '0' && args[1] == 0)
-		{
-			send_to_server("JOIN 0");
-			return;
-		}
-
-		/*
-		 * This gross hack is neccesary to make sure that if the
-		 * channel name or channel key contain double-quotes (which
-		 * is perfectly legal), that /window channel doesn't choke
-		 * all over them.
-		 */
-		*blah = '"';
-		double_quote(args, "\"", blah + 1);
-		strcpy(blah + strlen(blah), "\"");	/* Shoot me. */
-
-		window_channel(current_window, &blah);
+		window_rejoin(current_window, &args);
 		update_all_status();
 	}
 	else
@@ -713,8 +697,8 @@ static	void	e_pause_cb (char *u1, char *u2) { e_pause_cb_throw--; }
 BUILT_IN_COMMAND(e_pause)
 {
 	char *	sec;
-	int 	seconds;
-	time_t 	start;
+	double 	seconds;
+	struct timeval 	start;
 
 	if (!(sec = next_arg(args, &args)))
 	{
@@ -728,15 +712,16 @@ BUILT_IN_COMMAND(e_pause)
 		return;
 	}
 
-	seconds = my_atol(sec);
-	time(&start);
+	seconds = atof(sec);
+	get_time(&start);
+	start = time_add(start, double_to_timeval(seconds));
 
 	/* 
 	 * I use comment here simply becuase its not going to mess
 	 * with the arguments.
 	 */
 	add_timer(0, "", seconds, 1, (int (*)(void *))comment, NULL, NULL, current_window);
-	while (time(NULL) < start + seconds)
+	while (time_diff(get_time(NULL), start) > 0)
 		io("e_pause");
 }
 
@@ -748,7 +733,7 @@ BUILT_IN_COMMAND(e_privmsg)
 {
 	char	*nick;
 
-	if ((nick = next_arg(args, &args)) != NULL)
+	if ((nick = new_next_arg(args, &args)) != NULL)
 	{
 		if (!strcmp(nick, "."))
 		{
@@ -929,6 +914,21 @@ BUILT_IN_COMMAND(echocmd)
 			break;
 		    }
 
+		    case 'V':	/* VISUAL (output to a visible window) */
+		    {
+			/* There is always at least one visible window! */
+			Window *win = NULL;
+			while ((traverse_all_windows(&win)))
+			{
+				if (win->visible)
+				{
+					to_window = win;
+					break;
+				}
+			}
+			break;
+		    }
+
 		    case 'W':	/* WINDOW (output to specified window) */
 		    {
 			flag_arg = next_arg(args, &args);
@@ -1147,7 +1147,7 @@ BUILT_IN_COMMAND(flush)
 		flush_everything_being_held(NULL);
 
 	say("Standby, Flushing server output...");
-	flush_server();
+	flush_server(from_server);
 	say("Done");
 }
 
@@ -1258,12 +1258,16 @@ BUILT_IN_COMMAND(e_hostname)
 	{
 		struct hostent *hp;
 
-		malloc_strcpy(&LocalHostName, args);
-
-		if ((hp = gethostbyname(LocalHostName)))
-			memmove((void *)&LocalHostAddr, hp->h_addr, sizeof(LocalHostAddr));
-
-		say("Local host name is now %s", LocalHostName);
+		if ((hp = gethostbyname(args)))
+		{
+			malloc_strcpy(&LocalHostName, args);
+			memmove((void *)&LocalHostAddr, hp->h_addr, 
+					sizeof(LocalHostAddr));
+			say("Local host name is now %s", LocalHostName);
+		}
+		else
+			say("Cannot set Local host name to invalid host [%s]!",
+					args);
 	}
 	else
 		say("Local Host name is %s",
@@ -2051,6 +2055,8 @@ BUILT_IN_COMMAND(query)
 BUILT_IN_COMMAND(quotecmd)
 {
 	int	old_from_server = from_server;
+	int	urlencoded = 0;
+	size_t	length;
 
 	if (*command == 'X')
 	{
@@ -2063,6 +2069,10 @@ BUILT_IN_COMMAND(quotecmd)
 				if (is_server_connected(sval))
 					from_server = sval;
 			}
+			else if (!my_strnicmp(flag + 1, "U", 1)) /* URL quoting */
+			{
+				urlencoded++;
+			}
 			else if (!my_strnicmp(flag + 1, "A", 1)) /* ALL */
 			{
 				int	i;
@@ -2077,13 +2087,23 @@ BUILT_IN_COMMAND(quotecmd)
 						}
 					}
 				}
-				from_server = old_from_server;
-				return;
+				goto end;
+			}
+			else
+			{
+				/* End option processing on unknown arg. */
+				break;
 			}
 		}
 	}
 
-	if (args && *args)
+	if (urlencoded)
+	{
+		length = strlen(args);
+		urldecode(args, &length);
+		send_to_server_raw(length, args);
+	}
+	else if (args && *args)
 	{
 		char	*comm = new_next_arg(args, &args);
 		protocol_command *p;
@@ -2102,8 +2122,7 @@ BUILT_IN_COMMAND(quotecmd)
 		if (cnt < 0 && (rfc1459[loc].flags & PROTO_NOQUOTE))
 		{
 			yell("Doing /QUOTE %s is not permitted.  Use the client's built in command instead.", comm);
-			from_server = old_from_server;
-			return;
+			goto end;
 		}
 
 		/*
@@ -2116,6 +2135,7 @@ BUILT_IN_COMMAND(quotecmd)
 		send_to_server("%s %s", comm, args);
 	}
 
+end:
 	from_server = old_from_server;
 }
 
@@ -2148,9 +2168,13 @@ BUILT_IN_COMMAND(reconnect_cmd)
 		say("Reconnect to what server? (You're not connected)");
 		return;
 	}
+	if (!args || !*args)
+		args = "Reconnecting";
+
         say("Reconnecting to server %d", from_server);
+	set_server_quit_message(from_server, args);
 	server_reconnects_to(from_server, from_server);
-	reconnect(from_server);
+	reconnect(from_server, 1);
 }
 /* End of contributed code */
 
@@ -2432,9 +2456,17 @@ BUILT_IN_COMMAND(setenvcmd)
 	char *env_var;
 
 	if ((env_var = next_arg(args, &args)) != NULL)
-		bsd_setenv(env_var, args, 1);
+	{
+		if (*env_var == '-' && empty(args))
+			unsetenv(env_var + 1);
+		else 
+		{
+			unsetenv(env_var);
+			setenv(env_var, args, 1);
+		}
+	}
 	else
-		say("Usage: SETENV <var-name> <value>");
+		say("Usage: SETENV [-]<var-name> [<value>]");
 }
 
 BUILT_IN_COMMAND(shift_cmd)
@@ -2716,16 +2748,23 @@ int	command_exist (char *command)
 	return 0;
 }
 
-void	redirect_text (int to_server, const char *nick_list, const char *text, char *command, int hook)
+int	redirect_text (int to_server, const char *nick_list, const char *text, char *command, int hook)
 {
 static	int 	recursion = 0;
 	int 	old_from_server = from_server;
 	int	allow = 0;
+	int	retval;
 
 	from_server = to_server;
 	if (recursion++ == 0)
 		allow = do_hook(REDIRECT_LIST, "%s %s", nick_list, text);
 
+	/* Suppress output */
+	if (strcmp(nick_list, "0") == 0 || *nick_list == '@') 
+		retval = 1;
+	else
+		retval = 0;
+
 	/* 
 	 * Dont hook /ON REDIRECT if we're being called recursively
 	 */
@@ -2734,6 +2773,7 @@ static	int 	recursion = 0;
 
 	recursion--;
 	from_server = old_from_server;
+	return retval;
 }
 
 
@@ -2780,8 +2820,8 @@ void 	send_text (const char *nick_list, const char *text, const char *command, i
 		lastlog_level;
 	char 	*current_nick,
 		*next_nick,
-		*line,
-		*key;
+		*line;
+	Crypt	*key;
 	int	old_window_display = window_display;
 static	int	recursion = 0;
 
@@ -2836,13 +2876,25 @@ struct target_type target[4] =
 	     */
 	    else if (!text || !*text)
 		;
-#if 0
-	    /* This warning is useless now. */
+
+	    /*
+	     * Target 0 (an invalid irc target) is the "sink": messages
+	     * go in but they don't go out.
+	     */
+	    else if (!strcmp(current_nick, "0"))
+		;
+
+	    /*
+	     * Targets that start with @ are numbers that refer to 
+	     * $open() files.  This used to be used for DCC TALK but
+	     * we haven't supported that for 5 years.
+	     */
 	    else if (*current_nick == '@')
-		say("DCC TALK not supported.");
-#endif
+		target_file_write(current_nick + 1, text);
+
 	    else if (*current_nick == '"')
 		send_to_server("%s", text);
+
 	    else if (*current_nick == '/')
 	    {
 		line = m_opendup(current_nick, " ", text, NULL);
@@ -2971,6 +3023,35 @@ static void	eval_inputlist (char *args, char *line)
 	parse_line(NULL, args, line ? line : empty_string, 0, 0);
 }
 
+/*
+ * hrmm, these are remarkable similar to get_all_sets() and get_set() :)
+ */
+char *get_all_commands(void)
+{
+	int i;
+	char *ret = NULL;
+	size_t rclue = 0;
+
+	for (i = 0; irc_command[i].name; ++i)
+		m_sc3cat(&ret, space, irc_command[i].name, &rclue);
+	return ret;
+}
+
+char *get_command(const char *str)
+{
+	int i;
+	char *ret = NULL;
+	size_t rclue = 0;
+
+	if (!str || !*str)
+		return get_all_commands();
+	
+	for (i = 0; irc_command[i].name; ++i)
+		if (wild_match(str, irc_command[i].name))
+			m_sc3cat(&ret, space, irc_command[i].name, &rclue);
+
+	return ret ? ret : m_strdup(empty_string);
+}
 
 
 /* 
@@ -2999,7 +3080,7 @@ void 	command_completion (char unused, char *not_used)
 	char		buffer[BIG_BUFFER_SIZE + 1];
 
 	malloc_strcpy(&line, get_input());
-	if ((com = next_arg(line, &rest)) != NULL)
+	if (((com = next_arg(line, &rest)) != NULL) && *com)
 	{
 		if (!(cmdchars = get_string_var(CMDCHARS_VAR)))
 			cmdchars = DEFAULT_CMDCHARS;
@@ -3688,3 +3769,8 @@ BUILT_IN_COMMAND(returncmd)
 	}
 }
 
+BUILT_IN_COMMAND(allocdumpcmd)
+{
+	malloc_dump(args);
+}
+
