@@ -10,7 +10,7 @@
  */
 
 #if 0
-static	char	rcsid[] = "@(#)$Id: ctcp.c,v 1.16 1994/07/23 15:07:27 mrg Exp $";
+static	char	rcsid[] = "@(#)$Id: ctcp.c,v 1.10 2002/01/24 10:45:25 crazyed Exp $";
 #endif
 
 #include "irc.h"
@@ -31,6 +31,7 @@ static	char	rcsid[] = "@(#)$Id: ctcp.c,v 1.16 1994/07/23 15:07:27 mrg Exp $";
 #include "window.h"
 #include "if.h"
 #include "flood.h"
+#include "notice.h"
 
 #include <pwd.h>
 #ifdef HAVE_UNAME
@@ -156,8 +157,9 @@ int	in_ctcp_flag = 0;
  */
 CTCP_HANDLER(do_sed)
 {
-	char	*key = NULL,
-		*crypt_who;
+	Crypt	*key = NULL;
+	char	*crypt_who,
+		*tofrom;
 	char	*ret = NULL, *ret2 = NULL;
 
 	if (*from == '=')		/* DCC CHAT message */
@@ -167,18 +169,30 @@ CTCP_HANDLER(do_sed)
 	else
 		crypt_who = to;
 
-	if ((key = is_crypted(crypt_who)))
+	tofrom = m_3dup(to, ",", from);
+	m_3cat(&tofrom, "!", FromUserHost);
+
+	if ((key = is_crypted(tofrom)) ||
+	    (key = is_crypted(crypt_who)))
 		ret = decrypt_msg(cmd, key);
 
-	if (!key || !ret)
+	new_free(&tofrom);
+
+	if (!key || !ret) {
+		sed = 2;
 		malloc_strcpy(&ret2, "[ENCRYPTED MESSAGE]");
-	else
-	{
+	} else if (!*ret) {
+		sed = 2;
+		malloc_strcpy(&ret2, "[ENCRYPTED MESSAGE - BAD KEY?]");
+	} else {
 		/* 
 		 * There might be a CTCP message in there,
 		 * so we see if we can find it.
 		 */
-		ret2 = m_strdup(do_ctcp(from, to, ret));
+		if (doing_privmsg)
+			ret2 = m_strdup(do_ctcp(from, to, ret));
+		else if (doing_notice)
+			ret2 = m_strdup(do_notice_ctcp(from, to, ret));
 		sed = 1;
 	}
 
@@ -245,7 +259,7 @@ CTCP_HANDLER(do_dcc)
 	char	*size;
 	char	*extra_flags;
 
-	if (!is_me(from_server, to))
+	if (!is_me(from_server, to) && *from != '=')
 		return NULL;
 
 	if     (!(type = next_arg(cmd, &cmd)) ||
@@ -405,7 +419,7 @@ CTCP_HANDLER(do_finger)
 	else
 		my_host = hostname;
 
-	diff = time(NULL) - idle_time;
+	diff = time(NULL) - idle_time.tv_sec;
 
 	if (!(pwd = getpwuid(getuid())))
 		return NULL;
@@ -473,7 +487,7 @@ CTCP_HANDLER(do_ping_reply)
 	get_time(&t);
 
 	/* Reply must be between time we started and right now */
-	if (orig < start_time || orig > t.tv_sec)
+	if (orig < start_time.tv_sec || orig > t.tv_sec)
 	{
 		say("Invalid CTCP PING reply [%s] dropped.", cmd);
 		return NULL;
@@ -481,7 +495,7 @@ CTCP_HANDLER(do_ping_reply)
 
 	tsec = t.tv_sec - orig;
 
-	if ((ptr = strchr(cmd, ' ')))
+	if ((ptr = sindex(cmd, " .")))
 	{
 		*ptr++ = 0;
 		tusec = t.tv_usec - my_atol(ptr);
@@ -547,7 +561,7 @@ static	time_t	last_ctcp_parsed = 0;
 		allow_ctcp_reply = 0;	/* Historical limit of 4 CTCPs */
 
 	flag = check_ignore_channel(from, FromUserHost, to, IGNORE_CTCPS);
-	fflag = new_check_flooding(from, is_channel(to) ? to : NULL,
+	fflag = new_check_flooding(from, FromUserHost, is_channel(to) ? to : NULL,
 						str, CTCP_FLOOD); 
 
 	in_ctcp_flag++;
@@ -847,7 +861,7 @@ char *	do_notice_ctcp (char *from, char *to, char *str)
 		}
 
 		/* Toss it at the user.  */
-		if (do_hook(CTCP_REPLY_LIST, "%s %s %s", from, ctcp_command, ctcp_argument))
+		if (do_hook(CTCP_REPLY_LIST, "%s %s %s %s", from, to, ctcp_command, ctcp_argument))
 		{
 			/* Set up the window level/logging */
 			lastlog_level = set_lastlog_msg_level(LOG_CTCP);
