@@ -9,7 +9,7 @@
  */
 
 #if 0
-static	char	rcsid[] = "@(#)$Id: dcc.c,v 1.2 2000/12/07 18:24:43 jnelson Exp $";
+static	char	rcsid[] = "@(#)$Id: dcc.c,v 1.8 2002/01/10 02:26:30 crazyed Exp $";
 #endif
 
 #include "irc.h"
@@ -264,7 +264,6 @@ static 	void		dcc_erase (DCC_list *erased)
 		unsigned my_type = erased->flags & DCC_TYPES;
 		char	*dummy_ptr = NULL;
 		char 	*nopath;
-		char	*dummy_nick = NULL;
 		static time_t	last_reject = 0;
 		time_t	now;
 
@@ -280,13 +279,14 @@ static 	void		dcc_erase (DCC_list *erased)
 		else
 			nopath = erased->description;
 
+#if 0
 		if (*(erased->user) == '=')
 			dummy_nick = erased->user + 1;
 		else
 			dummy_nick = erased->user;
-
+#endif
 		dummy_ptr = m_sprintf("%s %s %s", 
-			dummy_nick,
+			erased->user,
 			(my_type == DCC_FILEOFFER ? "GET" :
 			 (my_type == DCC_FILEREAD  ? "SEND" : 
 			   dcc_types[my_type])),
@@ -297,7 +297,7 @@ static 	void		dcc_erase (DCC_list *erased)
 		/*
 		 * And output it to the user
 		 */
-		if (!dead)
+		if (!dead && *(erased->user) != '=')
 			isonbase(dummy_ptr, output_reject_ctcp);
 		else
 			output_reject_ctcp(dummy_ptr, erased->user);
@@ -664,11 +664,7 @@ static	int		dcc_open (DCC_list *dcc)
 	 */
 	else
 	{
-		unsigned short 	portnum1 = 0;
-		unsigned short	portnum2 = 0;
-
-		if (get_int_var(RANDOM_LOCAL_PORTS_VAR))
-			portnum1 = random_number(0) % (65536 - 1024) + 1024;
+		unsigned short	port = 0;
 
 		/*
 		 * Mark that we're waiting for the remote peer to answer,
@@ -679,12 +675,9 @@ static	int		dcc_open (DCC_list *dcc)
 		 * for a port if our random port isnt available.
 		 */
 		dcc->flags |= DCC_MY_OFFER;
-		if ((dcc->read = connect_by_number(NULL, &portnum1, 
-					SERVICE_SERVER, PROTOCOL_TCP)) < 0)
+		if ((dcc->read = connect_by_number(NULL, &port, 
+				SERVICE_SERVER, PROTOCOL_TCP)) < 0)
 		{
-		    if ((dcc->read = connect_by_number(NULL, &portnum2,
-					SERVICE_SERVER, PROTOCOL_TCP)) < 0)
-		    {
 			dcc->flags |= DCC_DELETE;
 			message_from(NULL, LOG_DCC);
 			say("Unable to create connection: %s", 
@@ -692,12 +685,8 @@ static	int		dcc_open (DCC_list *dcc)
 			message_from(NULL, LOG_CURRENT);
 			from_server = old_server;
 			return 0;
-		    }
-		    else
-			dcc->local_port = portnum2;
 		}
-		else
-		    dcc->local_port = portnum1;
+		dcc->local_port = port;
 
 #ifdef MIRC_BROKEN_DCC_RESUME
 		/*
@@ -1224,15 +1213,15 @@ static	void	dcc_getfile (char *args)
 /*
  * Calculates transfer speed based on size, start time, and current time.
  */
-static char *	calc_speed (off_t sofar, time_t sta, time_t cur)
+static char *	calc_speed (off_t sofar, struct timeval sta, struct timeval cur)
 {
 	static char	buf[7];
+	double		tdiff = time_diff(sta, cur);
 
-	if (sofar == 0 || (cur - sta) <= 0)
+	if (sofar == 0 || tdiff <= 0.0)
 		snprintf(buf, 7, "N/A");
 	else
-		snprintf(buf, 7, "%4.1f", 
-			(double)((double)sofar / (cur - sta) / 1024.0));
+		snprintf(buf, 7, "%4.1f", (double)sofar / tdiff / 1024.0);
 	return buf;
 }
 
@@ -1369,7 +1358,7 @@ static	char		*format =
 		if (Client->bytes_sent)
 		{
 			strlcpy(speed, calc_speed(act_sent, 
-				Client->starttime.tv_sec, time(NULL)), 9);
+				Client->starttime, get_time(NULL)), 9);
 		}
 		else
 			*speed = 0;
@@ -1927,12 +1916,19 @@ void	register_dcc_offer (char *user, char *type, char *description, char *addres
 	    if ((Client->flags & DCC_TYPES) == DCC_FILEREAD)
 	    {
 		struct stat statit;
+		char *realpath;
 
-		if (stat(Client->description, &statit) == 0)
+		if (get_string_var(DCC_STORE_PATH_VAR))
+		    realpath = m_sprintf("%s/%s", get_string_var(DCC_STORE_PATH_VAR), Client->description);
+		else
+		    realpath = m_strdup(Client->description);
+
+
+		if (stat(realpath, &statit) == 0)
 		{
 		    if (statit.st_size < Client->filesize)
 		    {
-			say("WARNING: File [%s] exists but is smaller then the file offered.", Client->description);
+			say("WARNING: File [%s] exists but is smaller then the file offered.", realpath);
 			say("Use /DCC CLOSE GET %s %s        to not get the file.", user, Client->description);
 #ifdef MIRC_BROKEN_DCC_RESUME
 			say("Use /DCC RESUME %s %s           to continue the copy where it left off.", user, Client->description);
@@ -1942,19 +1938,20 @@ void	register_dcc_offer (char *user, char *type, char *description, char *addres
 		    }
 		    else if (statit.st_size == Client->filesize)
 		    {
-			say("WARNING: File [%s] exists, and its the same size.", Client->description);
+			say("WARNING: File [%s] exists, and its the same size.", realpath);
 			say("Use /DCC CLOSE GET %s %s        to not get the file.", user, Client->description);
 			say("Use /DCC RENAME %s %s           to get the file anyways under a different name.", user, Client->description);
 			say("Use /DCC GET %s %s              to overwrite the existing file.", user, Client->description);
 		    }
 		    else 	/* Bigger than */
 		    {
-			say("WARNING: File [%s] already exists.", Client->description);
+			say("WARNING: File [%s] already exists.", realpath);
 			say("Use /DCC CLOSE GET %s %s        to not get the file.", user, Client->description);
 			say("Use /DCC RENAME %s %s           to get the file anyways under a different name.", user, Client->description);
 			say("Use /DCC GET %s %s              to overwrite the existing file.", user, Client->description);
 		    }
 		}
+		new_free(&realpath);
 	    }
 
 	    /* Thanks, Tychy! (lherron@imageek.york.cuny.edu) */
@@ -2007,6 +2004,7 @@ void	dcc_check (fd_set *Readables)
 
 		if (Readables && Client->read != -1 && FD_ISSET(Client->read, Readables))
 		{
+			FD_CLR(Client->read, Readables);	/* No more! */
 			previous_server = from_server;
 			from_server = -1;
 			message_from(NULL, LOG_DCC);
@@ -2037,8 +2035,8 @@ void	dcc_check (fd_set *Readables)
 		 * Enforce any timeouts
 		 */
 		else if (Client->flags & (DCC_MY_OFFER | DCC_THEIR_OFFER) &&
-				dcc_timeout >= 0 &&
-				(now - Client->lasttime.tv_sec > dcc_timeout))
+			 dcc_timeout >= 0 &&
+			 time_diff(Client->lasttime, now) > dcc_timeout)
 		{
 			Client->locked++;
 			if (do_hook(DCC_LOST_LIST,"%s %s %s IDLE TIME EXCEEDED",
@@ -2047,7 +2045,7 @@ void	dcc_check (fd_set *Readables)
 				Client->description ? 
 				 Client->description : "<any>"))
 			    say("Auto-rejecting a dcc after [%ld] seconds", 
-				now - Client->lasttime.tv_sec);
+				(long)time_diff(Client->lasttime, now));
 
 			/* 
 			 * This is safe to do after, since the connection
@@ -2196,7 +2194,7 @@ static	void	process_incoming_chat (DCC_list *Client)
 			/* See above for why these are safe. */
 			strcpy(tmp2, tmp);
 			strcpy(tmp, do_notice_ctcp(equal_nickname, nickname,
-				tmp2 + ((*tmp2 == CTCP_DELIM_CHAR) ? 0 : CTCP_MESSAGE_LEN)));
+				tmp2 + ((*tmp2 == CTCP_DELIM_CHAR) ? 0 : CTCP_REPLY_LEN)));
 			FromUserHost = empty_string;
 		}
 
@@ -2777,7 +2775,7 @@ static	char *	dcc_urldecode (const char *s)
 	char *str, *p1;
 
 	str = m_strdup(s);
-	urldecode(str);
+	urldecode(str, NULL);
 
 	for (p1 = str; *p1; p1++)
 	{
