@@ -477,7 +477,7 @@ say("Output from process %d (%s) now going to you", i, proc->name);
 			/*
 			 * Pretend to be just a dumb terminal
 			 */
-			bsd_setenv("TERM", "tty", 1);
+			setenv("TERM", "tty", 1);
 
 			/*
 			 * Figure out what shell (if any) we're using
@@ -591,7 +591,7 @@ say("Output from process %d (%s) now going to you", i, proc->name);
 			proc->redirect = NULL;
 			if (redirect)
 				proc->redirect = m_strdup(redirect);
-			proc->server = current_window->server;
+			proc->server = from_server;
 			proc->counter = 0;
 			proc->exited = 0;
 			proc->termsig = 0;
@@ -660,12 +660,18 @@ void 		do_processes (fd_set *rd)
 			continue;
 
 		if (proc->p_stdout != -1 && FD_ISSET(proc->p_stdout, rd))
+		{
+			FD_CLR(proc->p_stdout, rd);
 			handle_filedesc(proc, &proc->p_stdout, 
 					EXEC_PROMPT_LIST, EXEC_LIST);
+		}
 
 		if (proc->p_stderr != -1 && FD_ISSET(proc->p_stderr, rd))
+		{
+			FD_CLR(proc->p_stderr, rd);
 			handle_filedesc(proc, &proc->p_stderr,
 					EXEC_PROMPT_LIST, EXEC_ERRORS_LIST);
+		}
 
 		if (limit && proc->counter >= limit)
 			ignore_process(proc->index);
@@ -791,13 +797,10 @@ int 		get_child_exit (pid_t wanted)
 	 * or we've found the one asked for.
 	 */
 	if (dead_children_processes)
-	while ((pid = waitpid(wanted, &status, WNOHANG)) > 0)
 	{
-		/*
-		 * Ideally, this should never be < 0.
-		 */
-		dead_children_processes--;
-
+	    block_signal(SIGCHLD);
+	    while ((pid = waitpid(wanted, &status, WNOHANG)) > 0)
+	    {
 		/*
 		 * First thing we do is look to see if the process we're
 		 * working on is the one that was asked for.  If it is,
@@ -832,6 +835,9 @@ int 		get_child_exit (pid_t wanted)
 				}
 			}
 		}
+	    }
+	    dead_children_processes = 0;
+	    unblock_signal(SIGCHLD);
 	}
 
 	/*
