@@ -93,7 +93,7 @@ hook_functions[] =
 	{ "CHANNEL_SYNC",	(Hook *) 0,	3,	0,	0 },
 	{ "CONNECT",		(Hook *) 0,	2,	0,	0 },
 	{ "CTCP",		(Hook *) 0,	4,	0,	0 },
-	{ "CTCP_REPLY",		(Hook *) 0,	3,	0,	0 },
+	{ "CTCP_REPLY",		(Hook *) 0,	4,	0,	0 },
 	{ "CTCP_REQUEST",	(Hook *) 0,	4,	0,	0 },
 	{ "DCC_CHAT",		(Hook *) 0,	2,	0,	0 },
         { "DCC_CONNECT",        (Hook *) 0,     2,      0,      0 },
@@ -105,12 +105,15 @@ hook_functions[] =
 	{ "DISCONNECT",		(Hook *) 0,	1,	0,	0 },
         { "ENCRYPTED_NOTICE",   (Hook *) 0,     3,      0,      0 },
         { "ENCRYPTED_PRIVMSG",  (Hook *) 0,     3,      0,      0 },
+	{ "ERROR",		(Hook *) 0,	1,	0,	0 },
 	{ "EXEC",		(Hook *) 0,	2,	0,	0 },
 	{ "EXEC_ERRORS",	(Hook *) 0,	2,	0,	0 },
 	{ "EXEC_EXIT",		(Hook *) 0,	3,	0,	0 },
 	{ "EXEC_PROMPT",	(Hook *) 0,	2,	0,	0 },
         { "EXIT",               (Hook *) 0,     1,      0,      0 },
 	{ "FLOOD",		(Hook *) 0,	3,	0,	0 },
+	{ "GENERAL_NOTICE",	(Hook *) 0,	3,	0,	0 },
+	{ "GENERAL_PRIVMSG",	(Hook *) 0,	3,	0,	0 },
 	{ "HELP",		(Hook *) 0,	2,	0,	0 },
 	{ "HOOK",		(Hook *) 0,	1,	0,	0 },
 	{ "IDLE",		(Hook *) 0,	1,	0,	0 },
@@ -153,6 +156,7 @@ hook_functions[] =
 	{ "SET",		(Hook *) 0,	2,	0,	0 },
 	{ "SIGNOFF",		(Hook *) 0,	1,	0,	0 },
 	{ "SILENCE",		(Hook *) 0,	2,	0,	0 },
+	{ "SSL_SERVER_CERT",	(Hook *) 0,	3,	0,	0 },
 	{ "STATUS_UPDATE",	(Hook *) 0,	3,	0,	0 },
 	{ "SWITCH_CHANNELS",	(Hook *) 0,	3,	0,	0 },
 	{ "SWITCH_WINDOWS",	(Hook *) 0,	4,	0,	0 },
@@ -317,6 +321,7 @@ static void add_numeric_hook (int numeric, char *nick, char *stuff, int noisy, i
 		new_h = (Hook *)new_malloc(sizeof(Hook));
 		new_h->nick = NULL;
 		new_h->stuff = NULL;
+		new_h->filename = NULL;
 	}
 
 	malloc_strcpy(&new_h->nick, nick);
@@ -326,7 +331,7 @@ static void add_numeric_hook (int numeric, char *nick, char *stuff, int noisy, i
 	new_h->sernum = sernum;
 	new_h->flexible = flexible;
 	new_h->global = loading_global;
-	new_h->filename = m_strdup(current_package());
+	malloc_strcpy(&new_h->filename, current_package());
 	new_h->next = NULL;
 
 	upper(new_h->nick);
@@ -354,6 +359,7 @@ static void add_hook (int which, char *nick, char *stuff, int noisy, int not, in
 		new_h = (Hook *)new_malloc(sizeof(Hook));
 		new_h->nick = NULL;
 		new_h->stuff = NULL;
+		new_h->filename = NULL;
 	}
 
 	malloc_strcpy(&new_h->nick, nick);
@@ -363,7 +369,7 @@ static void add_hook (int which, char *nick, char *stuff, int noisy, int not, in
 	new_h->sernum = sernum;
 	new_h->flexible = flexible;
 	new_h->global = loading_global;
-	new_h->filename = m_strdup(current_package());
+	malloc_strcpy(&new_h->filename, current_package());
 	new_h->next = NULL;
 
 	upper(new_h->nick);
@@ -859,10 +865,17 @@ int 	do_hook (int which, char *format, ...)
 		name_copy = LOCAL_COPY(name);
 		stuff_copy = LOCAL_COPY(tmp->stuff);
 
-		if (tmp->noisy == UNKNOWN)
+		if (retval == RESULT_PENDING)
 		{
 			result = parse_line_with_return(name_copy, 
 						stuff_copy, buffer, 0, 0);
+
+			if (result && atol(result))
+				retval = SUPPRESS_DEFAULT;
+			else
+				retval = DONT_SUPPRESS_DEFAULT;
+
+			new_free(&result);
 		}
 		else
 		{
@@ -878,14 +891,6 @@ int 	do_hook (int which, char *format, ...)
 			return_exception = 0;
 		}
 
-		if (retval == RESULT_PENDING)
-		{
-			if (result && atol(result))
-				retval = SUPPRESS_DEFAULT;
-			else
-				retval = DONT_SUPPRESS_DEFAULT;
-		}
-
 		/*
 		 * Clean up the stuff that may have been mangled by the
 		 * execution.
