@@ -13,19 +13,19 @@
 /*
  * irc_version is what $J returns, its the common-name for the version.
  */
-const char irc_version[] = "EPIC4-1.0";
-const char useful_info[] = "epic4 1 0";
+const char irc_version[] = "EPIC4-1.1.2";
+const char useful_info[] = "epic4 1 1 2";
 
 /*
  * internal_version is what $V returns, its the integer-id for the
  * version, and corresponds to the date of release, YYYYMMDD.
  */ 
-const char internal_version[] = "20010401";
+const char internal_version[] = "20011203";
 
 /*
  * In theory, this number is incremented for every commit.
  */
-const unsigned long	commit_id = 71;
+const unsigned long	commit_id = 219;
 
 /*
  * As a way to poke fun at the current rage of naming releases after
@@ -33,7 +33,7 @@ const unsigned long	commit_id = 71;
  * reality, I have decided to start doing that with EPIC.  These names
  * are intentionally and maliciously silly.  Complaints will be ignored.
  */
-const char ridiculous_version_name[] = "Odyssey";
+const char ridiculous_version_name[] = "Ponderous";
 
 #define __need_putchar_x__
 #include "status.h"
@@ -109,10 +109,10 @@ int		quick_startup = 0;
 int		dont_connect = 0;
 
 /* Set to the current time, each time you press a key. */
-time_t		idle_time = 0;
+struct timeval	idle_time = { 0, 0 };
 
 /* Set to the time the client booted up */
-time_t		start_time;
+struct timeval	start_time;
 
 /* The number of child processes still unreaped. */
 int		child_dead = 0;
@@ -151,7 +151,7 @@ int		inhibit_logging = 0;
 int		loading_global = 0;
 
 /* This is reset every time io() is called.  Use this to save calls to time */
-time_t		now = 0;
+struct timeval	now = {0, 0};
 
 /*
  * If set, outbound connections will be bind()ed to the address
@@ -171,9 +171,10 @@ int		inbound_line_mangler = 0,
 		outbound_line_mangler = 0;
 
 char		*invite_channel = (char *) 0,	/* last channel of an INVITE */
-		*ircrc_file = (char *) 0,	/* full path .ircrc file */
+		*startup_file = NULL,		/* full path .epicrc file */
+		*epicrc_file = NULL,		/* full path .epicrc file */
+		*ircrc_file = NULL,		/* full path .ircrc file */
 		*my_path = (char *) 0,		/* path to users home dir */
-		*irc_path = (char *) 0,		/* paths used by /load */
 		*irc_lib = (char *) 0,		/* path to the ircII library */
 		*default_channel = NULL,	/* Channel to join on connect */
 		nickname[NICKNAME_LEN + 1],	/* users nickname */
@@ -236,6 +237,9 @@ void	irc_exit (int really_quit, char *format, ...)
 	char 	buffer[BIG_BUFFER_SIZE];
 	char *	sub_format;
 	int	old_window_display = window_display;
+#ifdef PERL
+	extern void perlstartstop(int);
+#endif
 
 	/*
 	 * If we get called recursively, something is hosed.
@@ -276,6 +280,9 @@ void	irc_exit (int really_quit, char *format, ...)
 
 	/* Do some clean up */
 	do_hook(EXIT_LIST, "%s", buffer);
+#ifdef PERL
+	perlstartstop(0);  /* In case there's perl code in the exit hook. */
+#endif
 	close_all_servers(buffer);
 	logger(0);
 	clean_up_processes();
@@ -320,7 +327,7 @@ volatile int	dead_children_processes;
  */
 SIGNAL_HANDLER(child_reap)
 {
-	dead_children_processes++;
+	dead_children_processes = 1;
 }
 
 volatile int	segv_recurse = 0;
@@ -351,22 +358,22 @@ to it, please dont worry about filling it out.  You might try talking	\n\
 to the person who is in charge of IRC at your site and see if you can	\n\
 get them to help you.							\n\
 									\n\
-This version of IRC II is  --->[%s]					\n\
+This version of IRC II is  --->[%s (%lu)]				\n\
 The date of release is     --->[%s]					\n\
 									\n\
 * * * * * * * * * * * * * * * * * * * * * * * *				\n\
-The program will now terminate.						\n", irc_version, internal_version);
+The program will now terminate.						\n", irc_version, commit_id, internal_version);
 
 		fflush(stdout);
 		panic_dump_call_stack();
 	}
 
         if (x_debug & DEBUG_CRASH)
-                irc_exit(0, "Hmmm. %s has another bug.  Go figure...",
-			irc_version);
+                irc_exit(0, "Hmmm. %s (%lu) has another bug.  Go figure...",
+			irc_version, commit_id);
         else
-                irc_exit(1, "Hmmm. %s has another bug.  Go figure...",
-			irc_version);
+                irc_exit(1, "Hmmm. %s (%lu) has another bug.  Go figure...",
+			irc_version, commit_id);
 }
 
 /*
@@ -425,7 +432,7 @@ SIGNAL_HANDLER(sig_user1)
 
 static	void	show_version (void)
 {
-	printf("ircII %s (Date of release: %s)\n\r",irc_version,internal_version);
+	printf("ircII %s (Commit id: %lu) (Date of release: %s)\n\r", irc_version, commit_id, internal_version);
 	exit (0);
 }
 
@@ -454,6 +461,8 @@ static	void	parse_args (int argc, char **argv)
 	struct hostent *hp;
 	char *ptr = (char *) 0;
 	char *tmp_hostname = NULL;
+	char *irc_path = NULL;
+	char *translation_path = NULL;
 
 	extern char *optarg;
 	extern int optind;
@@ -545,6 +554,11 @@ static	void	parse_args (int argc, char **argv)
 	if ((ptr = getenv("IRCPORT")))
 		irc_port = my_atol(ptr);
 
+	if ((ptr = getenv("EPICRC")))
+		epicrc_file = m_strdup(ptr);
+	else
+		epicrc_file = m_2dup(my_path, EPICRC_NAME);
+
 	if ((ptr = getenv("IRCRC")))
 		ircrc_file = m_strdup(ptr);
 	else
@@ -569,7 +583,13 @@ static	void	parse_args (int argc, char **argv)
 	if ((ptr = getenv("IRCHOST")) && *ptr)
 		tmp_hostname = ptr;
 
+	if ((ptr = getenv("IRCTRANSLATIONPATH")))
+		translation_path = m_strdup(ptr);
+	else
+		translation_path = m_2dup(IRCLIB, "/translation/");
 
+	set_string_var(TRANSLATION_PATH_VAR, translation_path);
+	new_free(&translation_path);
 
 	/*
 	 * Parse the command line arguments.
@@ -600,7 +620,7 @@ static	void	parse_args (int argc, char **argv)
 
 			case 'l': /* Load some file instead of ~/.ircrc */
 			case 'L': /* Same as above. Doesnt work like before */
-				malloc_strcpy(&ircrc_file, optarg);
+				malloc_strcpy(&epicrc_file, optarg);
 				break;
 
 			case 'a': /* add server, not replace */
@@ -674,7 +694,7 @@ static	void	parse_args (int argc, char **argv)
 
 	for (; *argv; argc--, argv++)
 		if (**argv)
-			build_server_list(*argv);
+			build_server_list(*argv, NULL);
 
 
 
@@ -689,7 +709,7 @@ static	void	parse_args (int argc, char **argv)
 	 * Find and build the server lists...
 	 */
 	if ((ptr = getenv("IRCSERVER")))
-		build_server_list(ptr);
+		build_server_list(ptr, NULL);
 
 	if (!server_list_size() || add_servers)
 	{
@@ -697,7 +717,7 @@ static	void	parse_args (int argc, char **argv)
 		if (!server_list_size())
 		{
 			ptr = m_strdup(DEFAULT_SERVER);
-			build_server_list(ptr);
+			build_server_list(ptr, NULL);
 			new_free(&ptr);
 		}
 	}
@@ -767,7 +787,7 @@ static	struct	timeval	clock_timeout,
 	static	int	last_warn = 0;
 
 	level++;
-	now = time(NULL);
+	get_time(&now);
 
 	/* Don't let this accumulate behind the user's back. */
 	cntl_c_hit = 0;
@@ -822,20 +842,20 @@ static	struct	timeval	clock_timeout,
 
 	/* CHECK FOR CPU SAVER MODE */
 	if (!cpu_saver && get_int_var(CPU_SAVER_AFTER_VAR))
-		if (now - idle_time > get_int_var(CPU_SAVER_AFTER_VAR) * 60)
+		if (now.tv_sec - idle_time.tv_sec > get_int_var(CPU_SAVER_AFTER_VAR) * 60)
 			cpu_saver_on(0, NULL);
 
 	/* SET UP FD SETS */
 	rd = readables;
 
-	clock_timeout.tv_sec = 60 - now % 60;
+	clock_timeout = time_to_next_minute();
 	if (cpu_saver && get_int_var(CPU_SAVER_EVERY_VAR))
 		clock_timeout.tv_sec += (get_int_var(CPU_SAVER_EVERY_VAR) - 1) * 60;
 
 	if (!timeptr)
 		timeptr = &clock_timeout;
-	timer.tv_sec = TimerTimeout();
-	if (timer.tv_sec <= timeptr->tv_sec)
+	timer = TimerTimeout();
+	if (time_diff(*timeptr, timer) < 0)
 		timeptr = &timer;
 
 	if ((hold_over = unhold_windows()))
@@ -848,7 +868,13 @@ static	struct	timeval	clock_timeout,
 	{
 		/* Timeout -- nothing interesting. */
 		case 0:
+		{
+#ifdef HAVE_SSL
+			/* Yes, this is slow, but we have to check for this */
+			do_server(&rd);
+#endif
 			break;
+		}
 
 		/* Interrupted system call -- check for SIGINT */
 		case -1:
@@ -903,7 +929,7 @@ static	struct	timeval	clock_timeout,
 
 	/* (set in term.c) -- we need to redraw the screen */
 	if (need_redraw)
-		refresh_screen('\0', NULL);
+		refresh_a_screen(main_screen);
 
 	window_check_channels();
 	alloca(0);
@@ -1140,7 +1166,7 @@ static		time_t	last_minute = -1;
 
 			from_server = primary_server;
 			do_hook(TIMER_LIST, "%02d:%02d", hour, min);
-			do_hook(IDLE_LIST, "%ld", (tv.tv_sec - idle_time) / 60);
+			do_hook(IDLE_LIST, "%ld", (tv.tv_sec - idle_time.tv_sec) / 60);
 			from_server = old_server;
 		}
 
@@ -1168,7 +1194,7 @@ int 	main (int argc, char *argv[])
 #ifdef SOCKS
 	SOCKSinit(argv[0]);
 #endif
-        start_time = time(NULL);
+        get_time(&start_time);
 	check_password();
 	check_valid_user();
 	check_invalid_host();
@@ -1177,7 +1203,7 @@ int 	main (int argc, char *argv[])
 
 	fprintf(stderr, "EPIC Version 4 -- %s\n", ridiculous_version_name);
 	fprintf(stderr, "EPIC Software Labs (2000)\n");
-	fprintf(stderr, "Version (%s) -- Date (%s)\n", irc_version, internal_version);
+	fprintf(stderr, "Version (%s), Commit Id (%lu) -- Date (%s)\n", irc_version, commit_id, internal_version);
 	fprintf(stderr, "%s\n", compile_info);
 	fprintf(stderr, "Process [%d]", getpid());
 	if (isatty(0))
@@ -1235,27 +1261,14 @@ int 	main (int argc, char *argv[])
 
 	/* XXXX Move this somewhere else eventually XXXX */
 	if (load_ircrc_right_away)
-	{
-		char	buffer[7];
-		strcpy(buffer, "global");
-
-		loading_global = 1;
-		load("LOAD", buffer, empty_string);
-		loading_global = 0;
-
-		/* read the .ircrc file */
-		if (access(ircrc_file, R_OK) == 0 && !quick_startup)
-			load("LOAD", ircrc_file, empty_string);
-
-		ircrc_loaded = 1;
-	}
+		load_ircrc();
 
 	if (dont_connect)
 		display_server_list();		/* Let user choose server */
 	else
-		reconnect(-1);			/* Connect to default server */
+		reconnect(-1, 0);		/* Connect to default server */
 
-	time(&idle_time);
+	get_time(&idle_time);
 	set_input(empty_string);
 	set_input_prompt(get_string_var(INPUT_PROMPT_VAR));
 	for (;;)
