@@ -8,18 +8,20 @@
  */
 
 #if 0
-static	char	rcsid[] = "@(#)$Id: ircaux.c,v 1.4 2001/01/23 19:00:27 jnelson Exp $";
+static	char	rcsid[] = "@(#)$Id: ircaux.c,v 1.32 2002/02/14 16:33:25 jnelson Exp $";
 #endif
 
 #include "irc.h"
 #include "screen.h"
 #include <pwd.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include "ircaux.h"
 #include "output.h"
 #include "term.h"
 #include "vars.h"
 #include "alias.h"
+#include "if.h"
 
 /*
  * This is the basic overhead for every malloc allocation (8 bytes).
@@ -32,8 +34,20 @@ typedef struct _mo_money
 {
 	unsigned magic;
 	int size;
+#ifdef ALLOC_DEBUG
+	unsigned entry;
+	char* fn;
+	int line;
+#endif
 } MO;
 
+#ifdef ALLOC_DEBUG
+struct {
+	int size;
+	void** entries;
+} alloc_table = { 0, NULL };
+#endif
+
 #define mo_ptr(ptr) ((MO *)( (char *)(ptr) - sizeof(MO) ))
 #define alloc_size(ptr) ((mo_ptr(ptr))->size)
 #define magic(ptr) ((mo_ptr(ptr))->magic)
@@ -57,6 +71,10 @@ static int	malloc_check (void *ptr)
 		return ALLOC_MAGIC_FAILED;
 	if (alloc_size(ptr) == FREED_VAL)
 		return ALREADY_FREED;
+#ifdef ALLOC_DEBUG
+	if (ptr != alloc_table.entries[mo_ptr(ptr)->entry])
+		return ALREADY_FREED;
+#endif
 	return NO_ERROR;
 }
 
@@ -116,6 +134,14 @@ void *	really_new_malloc (size_t size, char *fn, int line)
 	ptr += sizeof(MO);
 	magic(ptr) = ALLOC_MAGIC;
 	alloc_size(ptr) = size;
+#ifdef ALLOC_DEBUG
+	mo_ptr(ptr)->entry = alloc_table.size;
+	mo_ptr(ptr)->fn = fn;
+	mo_ptr(ptr)->line = line;
+	alloc_table.size++;
+	alloc_table.entries = realloc(alloc_table.entries, (alloc_table.size) * sizeof(void**));
+	alloc_table.entries[alloc_table.size-1] = ptr;
+#endif
 	return ptr;
 }
 
@@ -181,6 +207,11 @@ void *	really_new_free(void **ptr, char *fn, int line)
 	{
 		fatal_malloc_check(*ptr, NULL, fn, line);
 		alloc_size(*ptr) = FREED_VAL;
+#ifdef ALLOC_DEBUG
+		alloc_table.entries[mo_ptr(*ptr)->entry]
+			= alloc_table.entries[--alloc_table.size];
+		mo_ptr(alloc_table.entries[mo_ptr(*ptr)->entry])->entry = mo_ptr(*ptr)->entry;
+#endif
 #ifdef DELAYED_FREES
 		delay_free((void *)(mo_ptr(*ptr)));
 #else
@@ -190,9 +221,44 @@ void *	really_new_free(void **ptr, char *fn, int line)
 	return ((*ptr = NULL));
 }
 
+#if 1
+
+/* really_new_malloc in disguise */
+void *	really_new_realloc (void **ptr, size_t size, char *fn, int line)
+{
+	if (!size) 
+		*ptr = really_new_free(ptr, fn, line);
+	else if (!*ptr)
+		*ptr = really_new_malloc(size, fn, line);
+	else 
+	{
+		/* Make sure this is safe for realloc. */
+		fatal_malloc_check(*ptr, NULL, fn, line);
+
+		/* Copy everything, including the MO buffer */
+		if (!(*ptr = (char *)realloc(mo_ptr(*ptr), size + sizeof(MO))))
+			panic("realloc() failed, giving up!");
+
+		/* Re-initalize the MO buffer; magic(*ptr) is already set. */
+		*ptr += sizeof(MO);
+		alloc_size(*ptr) = size;
+#ifdef ALLOC_DEBUG
+		alloc_table.entries[mo_ptr(*ptr)->entry] = *ptr;
+#endif
+	}
+	return *ptr;
+}
+
+#else
+
 void *	new_realloc (void **ptr, size_t size)
 {
 	char *ptr2 = NULL;
+	size_t  foo,bar;
+
+	/* Yes the function is ifdefed out, but this serves as a proof of concept. */
+	for (foo=1, bar=size+sizeof(MO); bar; foo<<=1, bar>>=1) /* Nothing */ ;
+	if (foo) {size=foo;}
 
 	if (*ptr)
 	{
@@ -214,6 +280,26 @@ void *	new_realloc (void **ptr, size_t size)
 	return ((*ptr = ptr2));
 }
 
+#endif
+
+void malloc_dump (char* file) {
+#ifdef ALLOC_DEBUG
+	int	foo, bar;
+	FILE	*fd;
+
+	if (!(file && *file && (fd=fopen(file, "a"))))
+		fd=stdout;
+
+	for (foo = alloc_table.size; foo--;) {
+		fprintf(fd, "%s/%d\t%d\t", mo_ptr(alloc_table.entries[foo])->fn, mo_ptr(alloc_table.entries[foo])->line, mo_ptr(alloc_table.entries[foo])->size);
+		for (bar = 0; bar<mo_ptr(alloc_table.entries[foo])->size; bar++)
+			fprintf(fd, " %x", (unsigned char)(((char*)(alloc_table.entries[foo]))[bar]));
+		fprintf(fd, "\n");
+	}
+	fclose(fd);
+#endif
+}
+
 /*
  * malloc_strcpy:  Mallocs enough space for src to be copied in to where
  * ptr points to.
@@ -249,6 +335,7 @@ char *	malloc_strcpy (char **ptr, const char *src)
 char *	malloc_strcat (char **ptr, const char *src)
 {
 	size_t  msize;
+	size_t  psize;
 
 	if (*ptr)
 	{
@@ -258,9 +345,9 @@ char *	malloc_strcat (char **ptr, const char *src)
 		if (!src)
 			return *ptr;
 
-		msize = strlen(*ptr) + strlen(src) + 1;
+		msize = (psize=strlen(*ptr)) + strlen(src) + 1;
 		RESIZE(*ptr, char, msize);
-		return strcat(*ptr, src);
+		return strcat(psize+*ptr, src)-psize;
 	}
 
 	return (*ptr = m_strdup(src));
@@ -346,40 +433,41 @@ char *	m_strdup (const char *str)
 	return strcpy(ptr, str);
 }
 
-char *	m_e3cat (char **one, const char *yes1, const char *yes2)
+char *	m_ec3cat (char **one, const char *yes1, const char *yes2, size_t *clue)
 {
 	if (*one && **one)
-		return m_3cat(one, yes1, yes2);
+		return m_c3cat(one, yes1, yes2, clue);
 
 	return (*one = m_2dup(yes1, yes2));
 }
 
 
-char *	m_s3cat (char **one, const char *maybe, const char *definitely)
+char *	m_sc3cat (char **one, const char *maybe, const char *definitely, size_t *clue)
 {
 	if (*one && **one)
-		return m_3cat(one, maybe, definitely);
+		return m_c3cat(one, maybe, definitely, clue);
 
 	return malloc_strcpy(one, definitely);
 }
 
-char *	m_s3cat_s (char **one, const char *maybe, const char *ifthere)
+char *	m_sc3cat_s (char **one, const char *maybe, const char *ifthere, size_t *clue)
 {
 	if (ifthere && *ifthere)
-		return m_3cat(one, maybe, ifthere);
+		return m_c3cat(one, maybe, ifthere, clue);
 
 	return *one;
 }
 
-char *	m_3cat(char **one, const char *two, const char *three)
+char *	m_c3cat(char **one, const char *two, const char *three, size_t *clue)
 {
-	int 	msize = 0;
+	size_t	csize = clue?*clue:0;
+	int 	msize = csize;
 
 	if (*one)
 	{
 		if (alloc_size(*one) == FREED_VAL)
 			panic("free()d pointer passed to m_3cat");
-		msize += strlen(*one);
+		msize += strlen(csize+*one);
 	}
 	if (two)
 		msize += strlen(two);
@@ -395,9 +483,11 @@ char *	m_3cat(char **one, const char *two, const char *three)
 		RESIZE(*one, char, msize + 1);
 
 	if (two)
-		strcat(*one, two);
+		strcat(csize + *one, two);
 	if (three)
-		strcat(*one, three);
+		strcat(csize + *one, three);
+	if (clue) 
+		*clue = msize;
 
 	return *one;
 }
@@ -541,16 +631,19 @@ char *	next_arg (char *str, char **new_ptr)
 	return str;
 }
 
-char *	remove_trailing_spaces (char *foo)
+char *	remove_trailing_spaces (char *foo, size_t *cluep)
 {
 	char *end;
+	size_t clue = cluep?*cluep:0;
 	if (!*foo)
 		return foo;
 
-	end = foo + strlen(foo) - 1;
+	end = clue + foo + strlen(clue + foo) - 1;
 	while (end > foo && my_isspace(*end))
 		end--;
 	end[1] = 0;
+	if (cluep) 
+		*cluep = end - foo;
 	return foo;
 }
 
@@ -558,19 +651,21 @@ char *	remove_trailing_spaces (char *foo)
  * yanks off the last word from 'src'
  * kinda the opposite of next_arg
  */
-char *	last_arg (char **src)
+char *	last_arg (char **src, size_t *cluep)
 {
 	char *ptr;
 
 	if (!src || !*src)
 		return NULL;
 
-	remove_trailing_spaces(*src);
-	ptr = *src + strlen(*src) - 1;
+	remove_trailing_spaces(*src, cluep);
+	ptr = *src + (cluep ? *cluep : 0);
+	ptr += strlen(ptr);
+	ptr -= 1;
 
 	if (*ptr == '"')
 	{
-		for (ptr--;;ptr--)
+		for (ptr--; ; ptr--)
 		{
 			if (*ptr == '"')
 			{
@@ -588,7 +683,7 @@ char *	last_arg (char **src)
 	}
 	else
 	{
-		for (;;ptr--)
+		for (; ; ptr--)
 		{
 			if (*ptr == ' ')
 				break;
@@ -597,16 +692,14 @@ char *	last_arg (char **src)
 		}
 	}
 
+	if (cluep) 
+		*cluep = ptr - *src;
+
 	if (ptr == *src)
-	{
-		ptr = *src;
 		*src = empty_string;
-	}
 	else
-	{
 		*ptr++ = 0;
-		remove_trailing_spaces(*src);
-	}
+
 	return ptr;
 }
 
@@ -795,6 +888,25 @@ char *	next_in_comma_list (char *str, char **after)
 	return str;
 }
 
+/* Find the next instance of 'what' that isn't backslashed. */
+char *	findchar (char *str, int what)
+{
+	char *p;
+
+	for (p = str; *p; p++)
+	{
+		if (p[0] == '\\' && p[1])
+			p++;
+		else if (p[0] == what)
+			break;
+	}
+
+	if (*p == what)
+		return p;
+	else
+		return NULL;
+}
+
 unsigned char stricmp_table [] = 
 {
 	0,	1,	2,	3,	4,	5,	6,	7,
@@ -1033,28 +1145,6 @@ char *	m_strcat_ues (char **dest, char *src, int unescape)
 	return *dest;
 }
 
-
-/*
- * scanstr: looks for an occurrence of str in source.  If not found, returns
- * 0.  If it is found, returns the position in source (1 being the first
- * position).  Not the best way to handle this, but what the hell 
- */
-int	scanstr (char *str, char *source)
-{
-	int	i,
-		max,
-		len;
-
-	len = strlen(str);
-	max = strlen(source) - len;
-	for (i = 0; i <= max; i++, source++)
-	{
-		if (!my_strnicmp(source, str, len))
-			return (i + 1);
-	}
-	return (0);
-}
-
 /* expand_twiddle: expands ~ in pathnames. */
 char *	expand_twiddle (char *str)
 {
@@ -1365,13 +1455,13 @@ static	int recursion = 0;		/* Recursion is bad */
 	term_reset();
 	fprintf(stderr, "An unrecoverable logic error has occured.\n");
 	fprintf(stderr, "Please fill out the BUG_FORM file, and include the following message:\n");
-	fprintf(stderr, "Panic: [%s:%s]\n", irc_version, buffer);
+	fprintf(stderr, "Panic: [%s (%lu):%s]\n", irc_version, commit_id, buffer);
 	panic_dump_call_stack();
 
 	if (x_debug & DEBUG_CRASH)
-		irc_exit(0, "EPIC Panic: %s:%s", irc_version, buffer);
+		irc_exit(0, "EPIC Panic: %s (%lu):%s", irc_version, commit_id, buffer);
 	else
-		irc_exit(1, "EPIC Panic: %s:%s", irc_version, buffer);
+		irc_exit(1, "EPIC Panic: %s (%lu):%s", irc_version, commit_id, buffer);
 }
 
 /* beep_em: Not hard to figure this one out */
@@ -1393,6 +1483,119 @@ int 	end_strcmp (const char *one, const char *two, int bytes)
 		return -1;
 }
 
+/*
+ * exec a program, given its arguments and input, return its entire output.
+ * on call, *len is the length of input, and on return, it is set to the
+ * length of the data returned.
+ *
+ * Reading is done more agressively than writing to keep the buffers
+ * clean, and the data flowing.
+ *
+ * Potential Bugs:
+ *   - If the program in question locks up for any reason, so will epic.
+ *     This can be fixed with an appropriate timeout in the select call.
+ *   - If the program in question outputs enough data, epic will run out
+ *     of memory and dump core.
+ *   - Although input and the return values are char*'s, they are only
+ *     treated as blocks of data, the size of *len.
+ *
+ * Special Note: stdin and stdout are not expected to be textual.
+ */
+char*	exec_pipe (char *executable, char *input, size_t *len, char**args)
+{
+	int 	pipe0[2] = {-1, -1};
+	int 	pipe1[2] = {-1, -1};
+	pid_t	pid;
+	char *	ret = NULL;
+	size_t	retlen = 0, rdpos = 0, wrpos = 0;
+	fd_set	rdfds, wrfds;
+	int	fdmax;
+
+	if (pipe(pipe0) || pipe(pipe1))
+	{
+		yell("Cannot open pipes for %s: %s",
+				executable, strerror(errno));
+		close(pipe0[0]);
+		close(pipe0[1]);
+		close(pipe1[0]);
+		close(pipe1[1]);
+		return ret;
+	}
+
+	switch (pid = fork())
+	{
+	case -1:
+		yell("Cannot fork for %s: %s", 
+				executable, strerror(errno));
+		return ret;
+	case 0:
+		dup2(pipe0[0], 0);
+		dup2(pipe1[1], 1);
+		close(pipe0[1]);
+		close(pipe1[0]);
+		close(2);	/* we dont want to see errors yet */
+		setuid(getuid());
+		setgid(getgid());
+		execvp(executable, args);
+#if 0
+		/*
+		 * OK, well the problem with this is that the message
+		 * is going to go out on stdout right, and where does
+		 * that end up?
+		 */
+		yell("Cannot exec %s: %s", 
+			executable, strerror(errno));
+#endif
+		_exit(0);
+	default :
+		close(pipe0[0]);
+		close(pipe1[1]);
+		FD_ZERO(&rdfds);
+		FD_ZERO(&wrfds);
+		FD_SET(pipe1[0], &rdfds);
+		FD_SET(pipe0[1], &wrfds);
+		fdmax = 1 + MAX(pipe1[0], pipe0[1]);
+		for (;;) {
+			fd_set RDFDS = rdfds;
+			fd_set WRFDS = wrfds;
+			int foo;
+			foo = select(fdmax, &RDFDS, &WRFDS, NULL, NULL);
+			if (-1 == foo) {
+				yell("Broken select call: %s", strerror(errno));
+				if (EINTR == errno)
+					continue;
+				break;
+			} else if (0 == foo) {
+				break;
+			}
+			if (FD_ISSET(pipe1[0], &RDFDS)) {
+				retlen = rdpos + 4096;
+				new_realloc((void**)&ret, retlen);
+				foo = read(pipe1[0], ret+rdpos, retlen-rdpos);
+				if (0 == foo)
+					break;
+				else if (0 < foo)
+					rdpos += foo;
+			} else if (FD_ISSET(pipe0[1], &WRFDS)) {
+				if (input && wrpos < *len)
+					foo = write(pipe0[1], input+wrpos, MIN(512, *len-wrpos));
+				else {
+					FD_CLR(pipe0[1], &wrfds);
+					close(pipe0[1]);
+				}
+				if (0 < foo)
+					wrpos += foo;
+			}
+		}
+		close(pipe0[1]);
+		close(pipe1[0]);
+		waitpid(pid, NULL, WNOHANG);
+		new_realloc((void**)&ret, rdpos);
+		break;
+	}
+	*len = rdpos;
+	return ret;
+}
 
 static FILE *	open_compression (char *executable, char *filename)
 {
@@ -1773,7 +1976,7 @@ int 	opento(const char *filename, int flags, off_t position)
 
 
 /* swift and easy -- returns the size of the file */
-off_t 	file_size (char *filename)
+off_t 	file_size (const char *filename)
 {
 	struct stat statbuf;
 
@@ -1783,8 +1986,16 @@ off_t 	file_size (char *filename)
 		return -1;
 }
 
+int	file_exists (const char *filename)
+{
+	if (file_size(filename) == -1)
+		return 0;
+	else
+		return 1;
+}
+
 /* Gets the time in second/usecond if you can,  second/0 if you cant. */
-struct timeval 	get_time(struct timeval *timer)
+struct timeval 	get_time (struct timeval *timer)
 {
 	static struct timeval timer2;
 
@@ -1826,6 +2037,57 @@ double 	time_diff (struct timeval one, struct timeval two)
 	return (double)td.tv_sec + ((double)td.tv_usec / 1000000.0);
 }
 
+struct timeval double_to_timeval (double x)
+{
+	struct timeval td;
+	time_t	s;
+
+	s = (time_t) x;
+	x = x - s;
+	x = x * 1000000;
+
+	td.tv_sec = s;
+	td.tv_usec = (long) x;
+	return td;
+}
+
+/* 
+ * calculates the time elapsed between 'one' and 'two' where they were
+ * gotten probably with a call to get_time.  'one' should be the older
+ * timer and 'two' should be the most recent timer.
+ */
+struct timeval 	time_subtract (struct timeval one, struct timeval two)
+{
+	struct timeval td;
+
+	td.tv_sec = two.tv_sec - one.tv_sec;
+	td.tv_usec = two.tv_usec - one.tv_usec;
+	if (td.tv_usec < 0)
+	{
+		td.tv_usec += 1000000;
+		td.tv_sec--;
+	}
+	return td;
+}
+
+/* 
+ * Adds the interval "two" to the base time "one" and returns it.
+ */
+struct timeval 	time_add (struct timeval one, struct timeval two)
+{
+	struct timeval td;
+
+	td.tv_usec = one.tv_usec + two.tv_usec;
+	td.tv_sec = one.tv_sec + two.tv_sec;
+	if (td.tv_usec > 1000000)
+	{
+		td.tv_usec -= 1000000;
+		td.tv_sec++;
+	}
+	return td;
+}
+
+
 char *	plural (int number)
 {
 	return (number != 1) ? "s" : empty_string;
@@ -1861,8 +2123,14 @@ char *	ltoa (long foo)
 char *	ftoa (double foo)
 {
 	static char buffer [BIG_BUFFER_SIZE + 1];
+	extern double fmod (double, double);
 
-	sprintf(buffer, "%f", foo);
+	if (get_int_var(FLOATING_POINT_MATH_VAR)) {
+		sprintf(buffer, "%.50g", foo);
+	} else {
+		foo -= fmod(foo, 1);
+		sprintf(buffer, "%.0f", foo);
+	}
 	return buffer;
 }
 
@@ -2023,6 +2291,42 @@ char *	chop_word (char *str)
 	return str;
 }
 
+char *	skip_spaces (char *str)
+{
+	while (str && *str && isspace(*str))
+		str++;
+	return str;
+}
+
+int	split_args (char *str, char **to, size_t maxargs)
+{
+	int	counter;
+	char *	ptr;
+
+	ptr = str;
+	for (counter = 0; counter < maxargs; counter++)
+	{
+		if (!ptr || !*ptr)
+			break;
+
+		ptr = skip_spaces(ptr);
+		if (*ptr == '{' || *ptr == '(')
+		{
+			if (counter > 0)
+				ptr[-1] = 0;
+			to[counter] = next_expr_with_type(&ptr, *ptr);
+		}
+		else
+			to[counter] = new_next_arg(ptr, &ptr);
+
+		/* Syntax error? abort immediately. */
+		if (to[counter] == NULL)
+			break;
+	}
+	to[counter] = NULL;
+	return counter;
+}
+
 int 	splitw (char *str, char ***to)
 {
 	int numwords = word_count(str);
@@ -2044,13 +2348,14 @@ char *	unsplitw (char ***container, int howmany)
 {
 	char *retval = NULL;
 	char **str = *container;
+	size_t clue = 0;
 
 	if (!str || !*str)
 		return NULL;
 
 	while (howmany)
 	{
-		m_s3cat(&retval, " ", *str);
+		if (*str && **str) m_sc3cat(&retval, " ", *str, &clue);
 		str++, howmany--;
 	}
 
@@ -2301,27 +2606,42 @@ char *	get_userhost (void)
 }
 
 
-int 	time_to_next_minute (void)
+/* Fancy attempt to compensate for broken time_t's */
+struct timeval 	time_to_next_minute (void)
 {
 	static	int 	which = 0;
-	time_t 		now   = time(NULL);
+	struct timeval	now, then;
 
-	if (which == 1)
-		return 60 - now % 60;
-	else 
+	get_time(&now);
+
+	/* 
+	 * The first time called, try to determine if the system clock
+	 * is an exact multiple of 60 at the top of every minute.  If it
+	 * is, then we will use the "60 trick" to optimize calculations.
+	 * If it is not, then we will do it the hard time every time.
+	 */
+	if (which == 0)
 	{
-		struct tm *now_tm = gmtime(&now);
+		struct tm *now_tm = gmtime(&now.tv_sec);
 
 		if (!which)
 		{
-			if (now_tm->tm_sec == now % 60)
+			if (now_tm->tm_sec == now.tv_sec % 60)
 				which = 1;
 			else
 				which = 2;
 		}
-
-		return 60 - now_tm->tm_sec;
 	}
+
+	then.tv_usec = 1000000 - now.tv_usec;
+	if (which == 1)
+		then.tv_sec = 60 - (now.tv_sec + 1) % 60;
+	else 	/* which == 2 */
+	{
+		struct tm *now_tm = gmtime(&now.tv_sec);
+		then.tv_sec = 60 - (now_tm->tm_sec + 1) % 60;
+	}
+	return then;
 }
 
 /*
@@ -2414,8 +2734,8 @@ char *	encode(const char *str, int len)
 
 	while (len)
 	{
-		*ptr++ = (*str >> 4) + 0x41;
-		*ptr++ = (*str & 0x0f) + 0x41;
+		*ptr++ = ((unsigned char)*str >> 4) + 0x41;
+		*ptr++ = ((unsigned char)*str & 0x0f) + 0x41;
 		str++;
 		len--;
  	}
@@ -2460,19 +2780,12 @@ char *	chomp (char *s)
 /*
  * figure_out_address -- lets try this one more time.
  */
-int	figure_out_address (char *nuh, char **nick, char **user, char **host, char **domain, int *ip)
+int	figure_out_address (char *nuh, char **nick, char **user, char **host)
 {
 static 	char 	*mystuff = NULL;
-	char 	*firstback, 
-		*secondback, 
-		*thirdback, 
-		*fourthback;
 	char 	*bang, 
 		*at, 
-		*dot = NULL,
-		*myhost = star, 
-		*endstring;
-	int	number;
+		*dot = NULL;
 
 	/* Dont bother with channels, theyre ok. */
 	if (*nuh == '#' || *nuh == '&')
@@ -2480,8 +2793,7 @@ static 	char 	*mystuff = NULL;
 
 	malloc_strcpy(&mystuff, nuh);
 
-	*host = *domain = star;
-	*ip = 0;
+	*host = star;
 
 
 	/*
@@ -2522,6 +2834,9 @@ static 	char 	*mystuff = NULL;
 
 /*
  * STAGE ONE -- EXTRACT THE NICK, USER, AND HOST PORTIONS.
+ *
+ * stage two is now in 'figure_out_domain', so functions which want it
+ * that way can have it that way.
  */
 
 	/*
@@ -2534,7 +2849,7 @@ static 	char 	*mystuff = NULL;
 			/* nick!user@host */
 			*nick = mystuff;
 			*user = bang + 1;
-			myhost = at + 1;
+			*host = at + 1;
 		}
 		else
 		{
@@ -2542,13 +2857,13 @@ static 	char 	*mystuff = NULL;
 			{
 				*nick = mystuff;
 				*user = star;
-				myhost = at + 1;
+				*host = at + 1;
 			}
 			else			/* nick!user */
 			{
 				*nick = mystuff;
 				*user = star;
-				myhost = star;
+				*host = star;
 			}
 		}
 	}
@@ -2559,7 +2874,7 @@ static 	char 	*mystuff = NULL;
 			/* user@host.domain */
 			*nick = star;
 			*user = mystuff;
-			myhost = at + 1;
+			*host = at + 1;
 		}
 		else
 		{
@@ -2567,36 +2882,58 @@ static 	char 	*mystuff = NULL;
 			{
 				*nick = star;
 				*user = star;
-				myhost = mystuff;
+				*host = mystuff;
 			}
 			else			/* nick */
 			{
 				*nick = mystuff;
 				*user = star;
-				myhost = star;
+				*host = star;
+			}
 			}
 		}
-	}
 
+	return 0;
+}
+
+
+int	figure_out_domain (char *fqdn, char **host, char **domain, int *ip)
+{
+	char 	*firstback, 
+		*secondback, 
+		*thirdback, 
+		*fourthback;
+	char	*endstring;
+	char	*dot;
+	int	number;
+
+	/* determine if we have an IP, use dot to hold this */
+	/* is_number is better than my_atol since floating point
+	 * base 36 numbers are pretty much invalid as IPs.
+	 */
+	if ((dot = strrchr(fqdn, '.')) && is_number(dot + 1))
+		*ip = 1;
+	else
+		*ip = 0;
 
 /*
- * STAGE TWO -- EXTRACT THE HOST AND DOMAIN FROM MYHOST
+ * STAGE TWO -- EXTRACT THE HOST AND DOMAIN FROM FQDN
  */
 
 	/*
-	 * At this point, 'myhost' points what what we think the hostname
+	 * At this point, 'fqdn' points what what we think the hostname
 	 * is.  We chop it up into discrete parts and see what we end up with.
 	 */
-	endstring = myhost + strlen(myhost);
-	firstback = strnrchr(myhost, '.', 1);
-	secondback = strnrchr(myhost, '.', 2);
-	thirdback = strnrchr(myhost, '.', 3);
-	fourthback = strnrchr(myhost, '.', 4);
+	endstring = fqdn + strlen(fqdn);
+	firstback = strnrchr(fqdn, '.', 1);
+	secondback = strnrchr(fqdn, '.', 2);
+	thirdback = strnrchr(fqdn, '.', 3);
+	fourthback = strnrchr(fqdn, '.', 4);
 
 	/* Track foo@bar or some such thing. */
 	if (!firstback)
 	{
-		*host = myhost;
+		*host = fqdn;
 		return 0;
 	}
 
@@ -2605,10 +2942,9 @@ static 	char 	*mystuff = NULL;
 	 */
 	if (my_atol(firstback + 1))
 	{
-		*ip = 1;
-		*domain = myhost;
+		*domain = fqdn;
 
-		number = my_atol(myhost);
+		number = my_atol(fqdn);
 		if (number < 128)
 			*host = thirdback;
 		else if (number < 192)
@@ -2628,7 +2964,7 @@ static 	char 	*mystuff = NULL;
 	 */
 	else if (secondback && (endstring - firstback == 4))
 	{
-		*host = myhost;
+		*host = fqdn;
 		*domain = secondback;
 		**domain = 0;
 		(*domain)++;
@@ -2642,7 +2978,7 @@ static 	char 	*mystuff = NULL;
 			!strncmp(thirdback, ".k12.", 5) &&
 			!strncmp(firstback, ".us", 3))
 	{
-		*host = myhost;
+		*host = fqdn;
 		*domain = fourthback;
 		**domain = 0;
 		(*domain)++;
@@ -2657,7 +2993,7 @@ static 	char 	*mystuff = NULL;
 			!strncmp(firstback, ".us", 3))
 	{
 		*host = empty_string;
-		*domain = myhost;
+		*domain = fqdn;
 	}
 	/*
 	 *	(*).(*.???.??)
@@ -2667,7 +3003,7 @@ static 	char 	*mystuff = NULL;
 			(endstring - firstback == 3) &&
 			(firstback - secondback == 4))
 	{
-		*host = myhost;
+		*host = fqdn;
 		*domain = thirdback;
 		**domain = 0;
 		(*domain)++;
@@ -2681,7 +3017,7 @@ static 	char 	*mystuff = NULL;
 		 	(firstback - secondback == 4))
 	{
 		*host = empty_string;
-		*domain = myhost;
+		*domain = fqdn;
 	}
 	/*
 	 *	(*).(*.??.??)
@@ -2691,7 +3027,7 @@ static 	char 	*mystuff = NULL;
 			(endstring - firstback == 3) &&
 			(firstback - secondback == 3))
 	{
-		*host = myhost;
+		*host = fqdn;
 		*domain = thirdback;
 		**domain = 0;
 		(*domain)++;
@@ -2705,7 +3041,7 @@ static 	char 	*mystuff = NULL;
 			(firstback - secondback == 3))
 	{
 		*host = empty_string;
-		*domain = myhost;
+		*domain = fqdn;
 	}
 	/*
 	 *	(*).(*.??)
@@ -2713,7 +3049,7 @@ static 	char 	*mystuff = NULL;
 	 */
 	else if (secondback && (endstring - firstback == 3))
 	{
-		*host = myhost;
+		*host = fqdn;
 		*domain = secondback;
 		**domain = 0;
 		(*domain)++;
@@ -2724,7 +3060,7 @@ static 	char 	*mystuff = NULL;
 	else
 	{
 		*host = empty_string;
-		*domain = myhost;
+		*domain = fqdn;
 	}
 
 	return 0;
@@ -2883,12 +3219,12 @@ size_t	mangle_line	(char *incoming, int how, size_t how_much)
 
 	if (stuff & MANGLE_ANSI_CODES)
 	{
-		/* strip_ansi can expand up to three times */
+		/* normalize_string can expand up to three times */
 		char *output;
 
-		strip_ansi_never_xlate = 1;	/* XXXXX */
-		output = strip_ansi(incoming);
-		strip_ansi_never_xlate = 0;	/* XXXXX */
+		normalize_never_xlate = 1;	/* XXXXX */
+		output = normalize_string(incoming, 1);	/* Should be ok */
+		normalize_never_xlate = 0;	/* XXXXX */
 		if (strlcpy(incoming, output, how_much) > how_much)
 			say("Mangle_line truncating results. #1 -- "
 				"Email jnelson@acronet.net [%d] [%d]",
@@ -3149,7 +3485,7 @@ char *	urlencode (const char *s)
 		  )					\
 )
 
-char *	urldecode (char *s)
+char *	urldecode (char *s, size_t *length)
 {
 	const char *p1;
 	char *	p2;
@@ -3161,16 +3497,17 @@ char *	urldecode (char *s)
 	if (!s || !*s)
 		return NULL;
 
-	len = strlen(s);
+	len = length ? *length : strlen(s);
 	retval = alloca(len + 1);
 
-	for (p1 = s, p2 = retval; *p1; p1++, p2++)
+	for (p1 = s, p2 = retval; len--; p1++, p2++)
 	{
-		if (*p1 == '%' &&
-		    ((p1[1] && ((val1 = XTOI(p1[1])) != -1)) &&
-		     (p1[2] && ((val2 = XTOI(p1[2])) != -1))))
+		if (*p1 == '%' && len >= 2 &&
+		    (((val1 = XTOI(p1[1])) != -1) &&
+		     ((val2 = XTOI(p1[2])) != -1)))
 		{
 			p1++, p1++;
+			len--, len--;
 			*p2 = (val1 << 4) | val2;
 		}
 		else
@@ -3178,7 +3515,9 @@ char *	urldecode (char *s)
 	}
 
 	*p2 = 0;
-	return strcpy(s, retval);
+	if (length)
+		*length = p2 - retval;
+	return memcpy(s, retval, p2 - retval + 1);
 }
 
 unsigned char isspace_table [256] = 
