@@ -8,29 +8,38 @@
  */
 
 #if 0
-static	char	rcsid[] = "@(#)$Id: ircsig.c,v 1.3 1994/07/02 02:32:13 mrg Exp $";
+static	char	rcsid[] = "@(#)$Id: ircsig.c,v 1.2 2002/02/14 01:46:06 jnelson Exp $";
 #endif
 
 #include "irc.h"
 #include "irc_std.h"
 
-#ifdef HAVE_SIGACTION
-sigfunc *
-my_signal(sig_no, sig_handler, misc_flags)
-	int sig_no;
-	sigfunc *sig_handler;
-	int misc_flags;
+int	block_signal (int sig_no)
 {
-        /*
-         * misc_flags is unused currently.  it's planned to be used
-         * to use some of the doovier bits of sigaction(), if at
-         * some point we need them, -mrg
-         */
+	sigset_t set;
 
+	sigemptyset(&set);
+	sigaddset(&set, sig_no);
+	return sigprocmask(SIG_BLOCK, &set, NULL);
+}
+
+int	unblock_signal (int sig_no)
+{
+	sigset_t set;
+
+	sigemptyset(&set);
+	sigaddset(&set, sig_no);
+	return sigprocmask(SIG_UNBLOCK, &set, NULL);
+}
+
+sigfunc *my_signal (int sig_no, sigfunc *sig_handler)
+{
         struct sigaction sa, osa;
 
-        sa.sa_handler = sig_handler;
+	if (sig_no < 0)
+		return NULL;		/* Signal not implemented */
 
+        sa.sa_handler = sig_handler;
         sigemptyset(&sa.sa_mask);
         sigaddset(&sa.sa_mask, sig_no);
 
@@ -56,4 +65,3 @@ my_signal(sig_no, sig_handler, misc_flags)
 
         return (osa.sa_handler);
 }
-#endif /* USE_SIGACTION */
