@@ -72,7 +72,7 @@ struct timeval		join_time;	/* When we joined the channel */
  * The variable "mode_str" must correspond in order to the modes defined
  * here, or all heck will break loose.  You have been warned.
  */
-static	char	mode_str[] = "aciklmnprstzOR";
+static	char	mode_str[] = "aciklmnprstzMOR";
 
 const int	MODE_ANONYMOUS	= 1 << 0;	/* av2.9 */
 const int	MODE_C		= 1 << 1;	/* erf/TS4 */
@@ -86,8 +86,9 @@ const int	MODE_REGISTERED = 1 << 8;	/* Duhnet */
 const int	MODE_SECRET	= 1 << 9;	/* RFC */
 const int	MODE_TOPIC	= 1 << 10;	/* RFC */
 const int	MODE_Z		= 1 << 11;	/* erf/TS4 */
-const int	MODE_OPER_ONLY	= 1 << 12;	/* Duhnet */
-const int	MODE_RESTRICTED = 1 << 13;	/* Duhnet */
+const int	MODE_M          = 1 << 12;	/* Duhnet */
+const int	MODE_OPER_ONLY	= 1 << 13;	/* Duhnet */
+const int	MODE_RESTRICTED = 1 << 14;	/* Duhnet */
 
 
 /* channel_list: list of all the channels you are currently on */
@@ -794,6 +795,16 @@ int	is_chanvoice (const char *channel, const char *nick)
 		return 0;
 }
 
+int	is_halfop (const char *channel, const char *nick)
+{
+	Nick *n;
+
+	if ((n = find_nick(from_server, channel, nick)))
+		return n->half_assed;
+	else
+		return 0;
+}
+
 int	number_on_channel (const char *name, int server)
 {
 	Channel *channel = find_channel(name, server);
@@ -809,10 +820,11 @@ char	*create_nick_list (const char *name, int server)
 	Channel *channel = find_channel(name, server);
 	char 	*str = NULL;
 	int 	i;
+	size_t	clue = 0;
 
 	if (channel)
 		for (i = 0; i < channel->nicks.max; i++)
-			m_s3cat(&str, space, channel->nicks.list[i]->nick);
+			m_sc3cat(&str, space, channel->nicks.list[i]->nick, &clue);
 
 	return str;
 }
@@ -822,11 +834,12 @@ char	*create_chops_list (const char *name, int server)
 	Channel *channel = find_channel(name, server);
 	char 	*str = NULL;
 	int 	i;
+	size_t	clue = 0;
 
 	if (channel)
 		for (i = 0; i < channel->nicks.max; i++)
 			if (channel->nicks.list[i]->chanop)
-				m_s3cat(&str, space, channel->nicks.list[i]->nick);
+				m_sc3cat(&str, space, channel->nicks.list[i]->nick, &clue);
 
 	if (!str)
 		return m_strdup(empty_string);
@@ -838,11 +851,12 @@ char	*create_nochops_list (const char *name, int server)
 	Channel *channel = find_channel(name, server);
 	char 	*str = NULL;
 	int 	i;
+	size_t	clue = 0;
 
 	if (channel)
 		for (i = 0; i < channel->nicks.max; i++)
 			if (!channel->nicks.list[i]->chanop)
-				m_s3cat(&str, space, channel->nicks.list[i]->nick);
+				m_sc3cat(&str, space, channel->nicks.list[i]->nick, &clue);
 
 	if (!str)
 		return m_strdup(empty_string);
@@ -960,6 +974,9 @@ static void	decifer_mode (const char *modes, Channel *chan)
 			case 'z':		/* Erf/TS4 "zapped" */
 				value = MODE_Z;
 				break;
+			case 'M':		/* Duhnet's mute-mode */
+				value = MODE_M;
+				break;
 			case 'O':		/* Duhhnet's oper-only */
 				value = MODE_OPER_ONLY;
 				break;
@@ -970,7 +987,7 @@ static void	decifer_mode (const char *modes, Channel *chan)
 				if (!(key = next_arg(rest, &rest)))
 				{
 				    yell("Channel %s is +k, but has no key.  "
-					 "This server is broken", 
+					 "This server broke backwards compatability",
 					 chan->channel);
 				    continue;
 				}
@@ -1021,7 +1038,7 @@ static void	decifer_mode (const char *modes, Channel *chan)
 				{
 					yell("Channel %s got a mode +v "
 					     "without an argument.  "
-					     "This server is broken.",
+					     "This server broke backwards compatability",
 						chan->channel);
 					continue;
 				}
@@ -1038,7 +1055,7 @@ static void	decifer_mode (const char *modes, Channel *chan)
 				{
 					yell("Channel %s got a mode +h "
 					     "without an argument.  "
-					     "This server is broken.",
+					     "This server broke backwards compatability",
 						chan->channel);
 					continue;
 				}
@@ -1207,6 +1224,7 @@ char	*scan_channel (char *cname)
 	char		buffer[NICKNAME_LEN + 5];
 	char		*retval = NULL;
 	int		i;
+	size_t	clue = 0;
 
 	if (!wc)
 		return m_strdup(empty_string);
@@ -1216,6 +1234,8 @@ char	*scan_channel (char *cname)
 	{
 		if (nicks->list[i]->chanop)
 			buffer[0] = '@';
+		else if (nicks->list[i]->half_assed)
+			buffer[0] = '%';
 		else
 			buffer[0] = '.';
 
@@ -1227,7 +1247,7 @@ char	*scan_channel (char *cname)
 			buffer[1] = '.';
 
 		strlcpy(buffer + 2, nicks->list[i]->nick, NICKNAME_LEN);
-		m_s3cat(&retval, space, buffer);
+		m_sc3cat(&retval, space, buffer, &clue);
 	}
 
 	if (retval == NULL)
@@ -1320,6 +1340,22 @@ void 	change_server_channels (int old_s, int new_s)
 	}
 }
 
+/*
+ * This is called whenever you're not going to reconnect and
+ * destroy_server_channels() is called.
+ */
+void	destroy_waiting_channels (int server)
+{
+	Window *tmp = NULL;
+
+	while (traverse_all_windows(&tmp))
+	{
+		if (tmp->server != server)
+			continue;
+		new_free(&tmp->waiting_channel);
+	}
+}
+		
 /*
  * This is called by connect_to_new_server(), if the new server we are going
  * to attach to is already an established connection; by close_server(), if
@@ -1372,6 +1408,13 @@ void 	reconnect_all_channels (void)
 	char	*channels = NULL;
 	char	*keyed_channels = NULL;
 	char	*keys = NULL;
+	size_t	chan_clue = 0, kc_clue = 0, key_clue = 0;
+
+	/* Oh, what the heck. */
+	if (!get_int_var(AUTO_REJOIN_CONNECT_VAR)) {
+		destroy_server_channels(from_server);
+		return;
+	}
 
 	while (traverse_all_channels(&tmp, from_server))
 	{
@@ -1382,11 +1425,11 @@ void 	reconnect_all_channels (void)
 
 		if (tmp->key)
 		{
-			m_s3cat(&keyed_channels, ",", tmp->channel);
-			m_s3cat(&keys, ",", tmp->key);
+			m_sc3cat(&keyed_channels, ",", tmp->channel, &kc_clue);
+			m_sc3cat(&keys, ",", tmp->key, &key_clue);
 		}
 		else
-			m_s3cat(&channels, ",", tmp->channel);
+			m_sc3cat(&channels, ",", tmp->channel, &chan_clue);
 
 		clear_channel(tmp);
 	}
@@ -1474,6 +1517,16 @@ int 	get_channel_oper (const char *channel, int server)
 		return 0;
 }
 
+int	get_channel_halfop (const char *channel, int server)
+{
+	Channel *chan;
+
+	if ((chan = find_channel(channel, server)))
+		return chan->half_assed;
+	else
+		return 0;
+}
+
 int 	get_channel_voice (const char *channel, int server)
 {
 	Channel *chan;
@@ -1555,29 +1608,36 @@ void   move_channel_to_window (const char *chan, Window *old_w, Window *new_w)
 	{
 	    if (tmp->window == old_w && !my_stricmp(chan, tmp->channel))
 	    {
+		/*
+		 * First off, if this channel is bound to this window,
+		 * we gotta unbind it or there's going to be trouble
+		 * later on...
+		 */
+		if (old_w->bind_channel && !my_stricmp(chan, old_w->bind_channel))
+			new_free(&old_w->bind_channel);
+
 		if (!my_stricmp(chan, old_w->current_channel))
 		{
 			new_free(&old_w->current_channel);
 			old_w->update |= UPDATE_STATUS;
 
-			    if (new_w->current_channel)
+			if (new_w->current_channel)
 				old_chan = m_strdup(new_w->current_channel);
-			    else
+			else
 				old_chan = NULL;
 
-			    malloc_strcpy(&new_w->current_channel, chan);
+			malloc_strcpy(&new_w->current_channel, chan);
 
-			    /* 
-			     * Remove "waiting_channel" if we're waiting for 
-			     * this channel. ;-) 
-			     */
-			    if (new_w->waiting_channel && 
-				!my_stricmp(chan, new_w->waiting_channel))
+			/* 
+			 * Remove "waiting_channel" if we're waiting for 
+			 * this channel. ;-) 
+			 */
+			if (new_w->waiting_channel && 
+			    !my_stricmp(chan, new_w->waiting_channel))
 				    new_free(&new_w->waiting_channel);
 
-			    new_w->update |= UPDATE_STATUS;
-			    set_channel_window(new_w, new_w->current_channel);
-
+			new_w->update |= UPDATE_STATUS;
+			set_channel_window(new_w, new_w->current_channel);
 			reset_old_w = 1;
 			break;
 		}
@@ -1698,11 +1758,12 @@ char *	create_channel_list (int server)
 {
 	Channel	*tmp = NULL;
 	char	*retval = NULL;
+	size_t	clue = 0;
 
 	if (server >= 0)
 	{
 		while (traverse_all_channels(&tmp, server))
-			m_s3cat(&retval, space, tmp->channel);
+			m_sc3cat(&retval, space, tmp->channel, &clue);
 	}
 
 	return retval ? retval : m_strdup(empty_string);
