@@ -345,6 +345,228 @@ int 	dgets (char *str, int des, int buffer)
 		return 0;
 }
 
+#ifdef HAVE_SSL
+
+int SSL_dgets (char *str, int des, int buffer, int buffersize, SSL* ssl_fd)
+{
+	int	cnt = 0, c;
+	MyIO	*ioe;
+	int	nbytes;
+
+	if (!io_rec)
+		init_io();
+
+	ioe = io_rec[des];
+
+	if (ioe == NULL)
+	{
+		ioe = io_rec[des] = (MyIO *)new_malloc(sizeof(MyIO));
+		ioe->buffer_size = IO_BUFFER_SIZE;
+		ioe->buffer = (char *)new_malloc(ioe->buffer_size + 2);
+		ioe->read_pos = ioe->write_pos = 0;
+		ioe->segments = 0;
+		ioe->error = 0;
+	}
+
+	if (ioe->read_pos == ioe->write_pos)
+	{
+		ioe->read_pos = ioe->write_pos = 0;
+		ioe->buffer[0] = 0;
+		ioe->segments = 0;
+	}
+
+	if (!strchr(ioe->buffer + ioe->read_pos, '\n'))
+	{
+		if (ioe->read_pos)
+		{
+			ov_strcpy(ioe->buffer, ioe->buffer + ioe->read_pos);
+			ioe->read_pos = 0;
+			ioe->write_pos = strlen(ioe->buffer);
+			ioe->segments = 1;
+		}
+
+		/*
+		 * Dont try to read into a full buffer.
+		 */
+		if (ioe->write_pos >= ioe->buffer_size)
+		{
+			yell("***XXX*** Buffer for des [%d] is filled! ***XXX***", des);
+			dgets_errno = ENOMEM; /* Cough */
+			return -1;
+		}
+
+#ifdef HAVE_USEFUL_FIONREAD
+		/*
+		 * Check to see if any bytes are ready.  If this fails,
+		 * then its almost always due to the filedesc being 
+		 * bogus.  Thats a fatal error.
+		 */
+		if (ioctl(des, FIONREAD, &nbytes) == -1)
+		{
+			*str = 0;
+			dgets_errno = errno;
+			return -1;
+		} 
+
+		/*
+		 * Check for a quasi-EOF condition.  If we get to this
+		 * point, then new_select() indicated that this fd is ready.
+		 * The fd is ready if either:
+		 *	1) A newline is in the buffer
+		 *	2) select(2) returned ready for the fd.
+		 *
+		 * If 1) is true, then write_pos will not be zero.  So we can
+		 * use that as a cheap way to check for #1.  If #1 is false,
+		 * then #2 must have been true, and if nbytes is 0, then 
+		 * that indicates an EOF condition.
+		 */
+		else if (!nbytes && ioe->write_pos == 0)
+		{
+			*str = 0;
+			dgets_errno = -1;	/* End of file */
+			return -1;
+		}
+
+		/*
+		 * nbytes shouldnt ever be zero at this point.  Id be
+		 * interested in knowing if it ever was.
+		 */
+		else if (nbytes)
+#else
+		nbytes == -1;
+#endif
+		{
+			/* Better safe than sorry... */
+			c = SSL_read(ssl_fd, ioe->buffer + ioe->write_pos,
+				ioe->buffer_size - ioe->write_pos);
+			if (x_debug & DEBUG_INBOUND) 
+			    yell("FD [%d], should [%d] did [%d]", 
+					des, nbytes, c);
+
+			if (c <= 0)
+			{
+				*str = 0;
+				dgets_errno = (c == 0) ? -1 : errno;
+				return -1;
+			}
+			ioe->buffer[ioe->write_pos + c] = 0;
+			ioe->write_pos += c;
+			ioe->segments++;
+		}
+
+#ifdef HAVE_USEFUL_FIONREAD
+		else
+		{
+			/*
+			 * At this point nbytes is 0, and it doesnt
+			 * appear the socket is at EOF or ready to read.
+			 * Very little to do at this point but force the
+			 * issue and figure out what the heck went wrong.
+			 */
+			struct timeval t = { 0, 0 };
+			fd_set testing;
+
+			FD_ZERO(&testing);
+			FD_SET(des, &testing);
+			switch (select(des + 1, &testing, NULL, NULL, &t))
+			{
+	case -1:	/* Serious error condition */
+	{
+		yell("Fatal error on des [%d]", des);
+		*str = 0;
+		dgets_errno = errno;
+		return -1;
+	}
+	case 0:		/* Not ready to read */
+	{
+		if (ioe->write_pos == 0)
+			panic("des [%d] isnt ready, nothing buffered", des);
+		else		/* Flush the buffer to see if that helps */
+		{
+			ioe->buffer[ioe->write_pos++] = '\n';
+			break;
+		}
+	}
+	case 1:		/* Supposedly ready to read */
+	{
+		c = SSL_read(ssl_fd, ioe->buffer + ioe->write_pos,
+			ioe->buffer_size - ioe->write_pos);
+		if (c == -1)				/* Error. */
+		{
+			*str = 0;
+			dgets_errno = errno;
+			return -1;
+		}
+		else if (c == 0)			/* EOF */
+		{
+			/* I hope this is safe... */
+			strcpy(str, ioe->buffer + ioe->read_pos);
+			ioe->read_pos = ioe->write_pos = 0;
+			ioe->buffer[0] = 0;
+			dgets_errno = -1;		/* EOF */
+			return 0;
+		}
+		else					/* Success */
+		{
+			ioe->buffer[ioe->write_pos + c] = 0;
+			ioe->write_pos += c;
+			ioe->segments++;
+			break;
+		}
+
+	}
+			}
+		}
+#endif
+	}
+
+	dgets_errno = 0;
+
+	/*
+	 * If the caller wants us to force line buffering, and if there
+	 * is no complete line, just stop right here.
+	 */
+	if (buffer && !strchr(ioe->buffer + ioe->read_pos, '\n'))
+	{
+		if (ioe->segments > MAX_SEGMENTS)
+		{
+			yell("***XXX*** Too many read()s on des [%d] without a newline! ***XXX***", des);
+			*str = 0;
+			dgets_errno = ECONNABORTED;
+			return -1;
+		}
+		return 0;
+	}
+
+	/*
+	 * Slurp up the data that is available into 'str'. 
+	 */
+	while (ioe->read_pos < ioe->write_pos)
+	{
+		if (((str[cnt] = ioe->buffer[ioe->read_pos++])) == '\n')
+			break;
+		cnt++;
+	}
+
+	/*
+	 * Terminate it
+	 */
+	str[cnt + 1] = 0;
+
+	/*
+	 * If we end in a newline, then all is well.
+	 * Otherwise, we're unbuffered, tell the caller.
+	 * The caller then would need to do a strlen() to get
+ 	 * the amount of data.
+	 */
+	if (str[cnt] == '\n')
+		return cnt;
+	else
+		return 0;
+}
+
+#endif
+
 static	int global_max_fd = -1;
 
 /*
@@ -353,6 +575,7 @@ static	int global_max_fd = -1;
  */
 int 	new_select (fd_set *rd, fd_set *wd, struct timeval *timeout)
 {
+static	int		polls = 0;
 	struct timeval	thetimeout;
 	struct timeval *newtimeout = &thetimeout;
 		int	i,
@@ -361,7 +584,16 @@ int 	new_select (fd_set *rd, fd_set *wd, struct timeval *timeout)
 
 
 	if (timeout)
+	{
 		thetimeout = *timeout;
+		if (timeout->tv_sec == 0 && timeout->tv_usec == 0)
+		{
+			if (polls++ > 10000)
+				panic("Stuck in a polling loop. Help!");
+		}
+		else
+			polls = 0;
+	}
 	else
 		newtimeout = NULL;
 
