@@ -10,7 +10,7 @@
  */
 
 #if 0
-static	char	rcsid[] = "$Id: numbers.c,v 1.13 2001/03/24 03:43:40 jnelson Exp $";
+static	char	rcsid[] = "$Id: numbers.c,v 1.19 2002/02/04 05:37:44 jnelson Exp $";
 #endif
 
 #include "irc.h"
@@ -155,16 +155,23 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		lastlog_level;
 	int	old_current_numeric = current_numeric;
 
+	/* There are no valid numerics that do not have arguments */
 	if (!ArgList[0] || !ArgList[1])
 		return;
 
 	user = (*ArgList[0]) ? ArgList[0] : NULL;
+	ArgList++;
 
 	lastlog_level = set_lastlog_msg_level(LOG_CRAP);
 	message_from(NULL, LOG_CRAP);
-	ArgList++;
+
 	current_numeric = -comm;	/* must be negative of numeric! */
 
+	/*
+	 * This first switch statement is only for those numerics which
+	 * require special handling by the client.  Numerics which require
+	 * no more than displaying a message are handled below and not here.
+	 */
 	switch (comm)
 	{
 	/*
@@ -215,7 +222,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		PasteArgs(ArgList, 2);
 
 		/* Must do these things before calling "display_msg" */
-		add_to_server_list(new_server, new_port, NULL, NULL, 0);
+		add_to_server_list(new_server, new_port, NULL, NULL, NULL, 0);
 		server_reconnects_to(old_server, from_server);
 		from_server = old_server;
 
@@ -229,6 +236,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		set_server_cookie(from_server, ArgList[0]);
 		break;
 
+	/* XXX Doesn't belong here */
 	case 301:		/* #define RPL_AWAY             301 */
         {
                 PasteArgs(ArgList, 1);
@@ -246,6 +254,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		ison_returned(from, ArgList);
 		break;
 
+	/* XXX Doesn't belong here */
 	case 311:		/* #define RPL_WHOISUSER        311 */
         {
                 PasteArgs(ArgList, 4);
@@ -258,6 +267,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
                 break;
         }
 
+	/* XXX Doesn't belong here */
 	case 312:		/* #define RPL_WHOISSERVER      312 */
 	{
 		if (do_hook(current_numeric, "%s %s %s %s", from, ArgList[0], ArgList[1], ArgList[2]))
@@ -266,6 +276,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		break;
 	}
 
+	/* XXX Doesn't belong here */
 	case 313:		/* #define RPL_WHOISOPERATOR    313 */
 	{
 		PasteArgs(ArgList, 1);
@@ -274,6 +285,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		break;
 	}
 
+	/* XXX Doesn't belong here */
 	case 314:		/* #define RPL_WHOWASUSER       314 */
 	{
 		PasteArgs(ArgList, 4);
@@ -293,9 +305,16 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		break;
 	}
 
+#if 0
+	/* 
+	 * At the specific request of Kev, don't just eat this, but
+	 * treat it as Just Another Numeric.
+	 */
 	case 316:		/* supported, but deprecated */
 		break;
+#endif
 
+	/* XXX Doesn't belong here */
 	case 317:		/* #define RPL_WHOISIDLE        317 */
 	{
 		char	flag, *nick, *idle_str, *startup_str;
@@ -353,6 +372,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		break;
 	}
 
+	/* XXX Doesn't belong here */
 	case 318:		/* #define RPL_ENDOFWHOIS       318 */
         {
                 PasteArgs(ArgList, 0);
@@ -362,6 +382,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
                 break;
         }
 
+	/* XXX Doesn't belong here */
 	case 319:		/* #define RPL_WHOISCHANNELS    319 */
 	{
 		PasteArgs(ArgList, 1);
@@ -371,12 +392,14 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 	}
 
 	case 321:		/* #define RPL_LISTSTART        321 */
+	{
 		ArgList[0] = "Channel\0Users\0Topic";
 		ArgList[1] = ArgList[0] + 8;
 		ArgList[2] = ArgList[1] + 6;
 		ArgList[3] = (char *) 0;
 		funny_list(from, ArgList);
 		break;
+	}
 
 	case 322:		/* #define RPL_LIST             322 */
 		funny_list(from, ArgList);
@@ -386,6 +409,12 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		funny_mode(from, ArgList);
 		break;
 
+#if 0
+	/* 
+	 * At the specific request of Kev, I am suspending support 
+	 * for this aircd feature, as it has been re-used by one of the
+	 * big four.  XXX - It didn't belong here anyways.
+	 */
 	case 340:		/* #define RPL_INVITING_OTHER	340 */
 	{
 		if (ArgList[2])
@@ -396,7 +425,9 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		}
 		break;
 	}
+#endif
 
+	/* XXX Doesn't belong here */
 	case 341:		/* #define RPL_INVITING         341 */
 	{
 		if (ArgList[1])
@@ -421,6 +452,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		xwhoreply(NULL, ArgList);
 		break;
 
+	/* XXX Doesn't belong here */
 	case 366:		/* #define RPL_ENDOFNAMES       366 */
 	{
 		int	flag = 1;
@@ -520,6 +552,8 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		set_server_motd(from_server, 0);
 		break;
 	}
+
+	/* XXX Shouldn't this set "You're operator" flag for hybrid? */
 	case 381: 		/* #define RPL_YOUREOPER        381 */
 		PasteArgs(ArgList, 0);
 		if (!is_server_connected(from_server))
@@ -550,14 +584,14 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		break;
 	}
 
-	/* Broken dalnet extended who replies. */
+	/* Bizarre dalnet extended who replies. */
         /* ":%s 402 %s %s :No such server" */
 	case 402:
 	{
 		PasteArgs(ArgList, 1);
 		/* 
-		 * Some broken servers send this instead of a 315 numeric
-		 * when a who request has been completed.
+		 * Some servers BBC by sending this instead of
+		 * a 315 numeric when a who request has been completed.
 		 */
 		fake_who_end(from, ArgList[0]);
 
@@ -567,7 +601,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		break;
 	}
 
-	/* Yet more broken dalnet extended who replies. */
+	/* Yet more bizarre dalnet extended who replies. */
 	/* ":%s 522 %s :/WHO Syntax incorrect, use /who ? for help" */
         /* ":%s 523 %s :Error, /who limit of %d exceed." */
 	case 522:
@@ -597,7 +631,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		if (my_strnicmp(s, from, strlen(s)))
 			break;
 
-		/* Some broken servers may send back an empty reply. */
+		/* Some servers BBC and send back an empty reply. */
 		if (!ArgList[0])
 		{
 			if (do_hook(current_numeric, "%s *", from))
@@ -607,8 +641,8 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		}
 
 		/* 
-		 * Some broken servers send this instead of a 315 numeric
-		 * when a who request has been completed.
+		 * Some servers BBC and send this instead of a
+		 * 315 numeric when a who request has been completed.
 		 */
 		if (fake_who_end(from, ArgList[0]))
 			;
@@ -646,7 +680,9 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 	}
 	case 432:		/* #define ERR_ERRONEUSNICKNAME 432 */
 	{
-		if (get_int_var(AUTO_NEW_NICK_VAR))
+		if (!my_stricmp(user, ArgList[0]))
+			yell("WARNING:  Strange invalid nick message received.  You are probably lagged.");
+		else if (get_int_var(AUTO_NEW_NICK_VAR))
 			fudge_nickname(from_server);
 		else
 			reset_nickname(from_server);
@@ -750,7 +786,10 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 	case 438:		/* EFnet/TS4 "nick collision" numeric 438 */
 	case 453:		/* EFnet/TS4 "nickname lost" numeric 453 */
 	{
-		if (get_int_var(AUTO_NEW_NICK_VAR))
+		if (!my_stricmp(user, ArgList[0]))
+			/* This should stop the "rolling nicks" in their tracks. */
+			yell("WARNING:  Strange invalid nick message received.  You are probably lagged.");
+		else if (get_int_var(AUTO_NEW_NICK_VAR))
 			fudge_nickname(from_server);
 		else
 			reset_nickname(from_server);
@@ -860,6 +899,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		break;
 	}
 
+	/* XXX These do not belong here */
 	case 219:		/* #define RPL_ENDOFSTATS       219 */
 	case 232:		/* #define RPL_ENDOFSERVICES    232 */
 	case 365:		/* #define RPL_ENDOFLINKS       365 */
@@ -885,6 +925,15 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 	{
 		char *reason;
 
+		/* ircnet has a different 477 numeric. */
+		if (comm == 477 && ArgList[0] && *ArgList[0] == '+')
+		{
+			PasteArgs(ArgList, 0);
+			if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
+				display_msg(from, ArgList);
+			break;
+		}
+
 		if (ArgList[0])
 			cant_join_channel(ArgList[0], from_server);
 
