@@ -43,10 +43,13 @@ void unflash (void)
 #endif
 }
 
-/* sig_refresh_screen: the signal-callable versino of refresh_screen */
+/* sig_refresh_screen: the signal-callable version of refresh_screen */
 SIGNAL_HANDLER(sig_refresh_screen)
 {
-	refresh_screen(0, NULL);
+	need_redraw = 1;
+#if 0
+	refresh_a_screen(main_screen);
+#endif
 }
 
 /*
@@ -54,11 +57,16 @@ SIGNAL_HANDLER(sig_refresh_screen)
  * swoops into effect 
  */
 void refresh_screen (char dumb, char *dumber)
+{
+	refresh_a_screen(current_window->screen);
+}
+
+void	refresh_a_screen (Screen *screen)
 {
 	unflash();
 	term_clear_screen();
 
-	if (term_resize())
+	if (screen != main_screen || term_resize())
 		recalculate_windows(current_window->screen);
 	else
 		redraw_all_windows();
@@ -180,6 +188,24 @@ void	put_it (const char *format, ...)
 	}
 }
 
+void	file_put_it (FILE *fp, const char *format, ...)
+{
+	if (format)
+	{
+		va_list args;
+		va_start (args, format);
+		vsnprintf(putbuf, OBNOXIOUS_BUFFER_SIZE, format, args);
+		va_end(args);
+		if (fp)
+		{
+			fputs(putbuf, fp);
+			fputs("\n", fp);
+		}
+		else if (window_display)
+			put_echo(putbuf);
+	}
+}
+
 /* 
  * This is an alternative form of put_it which writes three asterisks
  * before actually putting things out.
