@@ -200,7 +200,7 @@ static void p_topic (char *from, char **ArgList)
 			break;
 	}
 
-	if (!new_check_flooding(from, ArgList[0], ArgList[1], TOPIC_FLOOD))
+	if (!new_check_flooding(from, FromUserHost, ArgList[0], ArgList[1], TOPIC_FLOOD))
 		return;
 
 	message_from(ArgList[0], LOG_CRAP);
@@ -221,7 +221,7 @@ static void p_wallops (char *from, char **ArgList)
 	}
 
 	/* wallops from a server */
-	if (server_wallop || check_flooding(from, WALLOP_FLOOD, ArgList[0]))
+	if (server_wallop || check_flooding(from, FromUserHost, WALLOP_FLOOD, ArgList[0]))
 	{
 		int	level;
 		char	*high;
@@ -279,6 +279,7 @@ static void p_privmsg (char *from, char **Args)
 	}
 
 	doing_privmsg = 1;
+	sed = 0;
 
 	/*
 	 * Do ctcp's first, and if there's nothing left, then dont
@@ -347,19 +348,22 @@ static void p_privmsg (char *from, char **Args)
 			high = empty_string;
 			break;
 	}
-	flood = new_check_flooding(from, flood_channel, ptr, flood_type);
+	flood = new_check_flooding(from, FromUserHost, flood_channel, ptr, flood_type);
 
 	/* Encrypted privmsgs are specifically exempted from flood control */
 	level = set_lastlog_msg_level(log_type);
 	if (sed != 0)
 	{
-		sed = 0;
-		if (!do_hook(ENCRYPTED_PRIVMSG_LIST, "%s %s %s", from, to, ptr))
+		if (!do_hook(ENCRYPTED_PRIVMSG_LIST, "%s %s %s", from, to,
+				sed == 1 ? ptr : empty_string))
 			hook_normal = 0;
+		sed = 0;
 	}
 
 	if (flood && hook_normal)
 	{ 
+	  if (do_hook(GENERAL_PRIVMSG_LIST, "%s %s %s", from, to, ptr))
+	  {
 	    switch (list_type)
 	    {
 		case PUBLIC_MSG_LIST:
@@ -412,8 +416,9 @@ static void p_privmsg (char *from, char **Args)
 			break;
 		}
 	    }
-	    if (beep_on_level & log_type)
-		    beep_em(1);
+	  }
+	  if (beep_on_level & log_type)
+	    beep_em(1);
 	}
 
 	sed = 0;
@@ -424,7 +429,7 @@ static void p_privmsg (char *from, char **Args)
 
 static void p_quit (char *from, char **ArgList)
 {
-	int		one_prints = 0;
+	int		one_prints = 1;
 	const char *	chan;
 	char *		Reason;
 
@@ -434,9 +439,16 @@ static void p_quit (char *from, char **ArgList)
 		Reason = ArgList[0] ? ArgList[0] : "?";
 		for (chan = walk_channels(1, from); chan; chan = walk_channels(0, from))
 		{
+			if (check_ignore_channel(from, FromUserHost, chan,
+							IGNORE_CRAP) == IGNORED)
+			{       
+				one_prints = 0;
+				continue;
+			}
+
 			message_from(chan, LOG_CRAP);
-			if (do_hook(CHANNEL_SIGNOFF_LIST, "%s %s %s", chan, from, Reason))
-				one_prints = 1;
+			if (!do_hook(CHANNEL_SIGNOFF_LIST, "%s %s %s", chan, from, Reason))
+				one_prints = 0;
 			message_from((char *) 0, LOG_CURRENT);
 		}
 		if (one_prints)
@@ -568,7 +580,7 @@ static void	p_channel (char *from, char **ArgList)
 			break;
 	}
 
-	if (!new_check_flooding(from, channel, star, JOIN_FLOOD))
+	if (!new_check_flooding(from, FromUserHost, channel, star, JOIN_FLOOD))
 		return;
 
 	*extra = 0;
@@ -608,7 +620,7 @@ static void 	p_invite (char *from, char **ArgList)
 			break;
 	}
 
-	if (!check_flooding(from, INVITE_FLOOD, ArgList[1]))
+	if (!check_flooding(from, FromUserHost, INVITE_FLOOD, ArgList[1]))
 		return;
 
 	if (ArgList[0] && ArgList[1])
@@ -760,7 +772,7 @@ static void p_nick (char *from, char **ArgList)
 			break;
 	}
 
-	if (!check_flooding(from, NICK_FLOOD, line))
+	if (!check_flooding(from, FromUserHost, NICK_FLOOD, line))
 		goto do_rename;
 
 	for (chan = walk_channels(1, from); chan; chan = walk_channels(0, from))
@@ -862,6 +874,12 @@ static void strip_modes (char *from, char *channel, char *line)
 	    {
 		char	c = *pointer;
 
+		/* 
+		 * Conversion from "next_arg" to "safe_new_next_arg"
+		 * done on Aug 30, 2001 because of lame-o servers that
+		 * don't send arguments with +l, +b, +k, +o, +v, +e, +I
+		 * and the like.  You all know who you are.  Don't do that.
+		 */
 		switch (c) 
 		{
 			case '+' :
@@ -875,7 +893,7 @@ static void strip_modes (char *from, char *channel, char *line)
 					do_hook(MODE_STRIPPED_LIST,
 						"%s %s %c%c %s",
 						from, channel, mag,
-						c,next_arg(copy,&copy));
+						c,safe_new_next_arg(copy,&copy));
 				else
 					do_hook(MODE_STRIPPED_LIST,
 						"%s %s %c%c",
@@ -886,6 +904,7 @@ static void strip_modes (char *from, char *channel, char *line)
 			case 'a' : case 'i' : case 'm' : case 'n' :
 			case 'p' : case 's' : case 't' : case 'z' : 
 			case 'c' : case 'r' : case 'R' : case 'O' :
+			case 'M' :
 			{
 				do_hook(MODE_STRIPPED_LIST,
 					"%s %s %c%c",
@@ -899,7 +918,7 @@ static void strip_modes (char *from, char *channel, char *line)
 				do_hook(MODE_STRIPPED_LIST,
 					"%s %s %c%c %s", 
 					from, channel, mag, c,
-					next_arg(copy,&copy));
+					safe_new_next_arg(copy,&copy));
 				break;
 			}
 		}
