@@ -21,6 +21,10 @@
 #include "status.h"
 #include "vars.h"
 #include "newio.h"
+#include "translat.h"
+#ifdef HAVE_SSL
+#include "ssl.h"
+#endif
 
 /*
  * Note to future maintainers -- we do a bit of chicanery here.  The 'flags'
@@ -41,6 +45,7 @@ const 	char *	umodes = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
 static	char *	do_umode (int du_index);
 	void 	reset_nickname (int);
 	void	clear_reconnect_counts (void);
+	const char *get_server_group (int refnum);
 
 	int	never_connected = 1;		/* true until first connection
 						 * is made */
@@ -67,7 +72,7 @@ static	char *	do_umode (int du_index);
  * passes.  If the server is not on the list, it is added to the end. In
  * either case, the server is made the current server. 
  */
-void 	add_to_server_list (const char *server, int port, const char *password, const char *nick, int overwrite)
+void 	add_to_server_list (const char *server, int port, const char *password, const char *nick, const char *group, int overwrite)
 {
 	Server *s;
 
@@ -80,6 +85,7 @@ void 	add_to_server_list (const char *server, int port, const char *password, co
 		s->name = m_strdup(server);
 		s->itsname = (char *) 0;
 		s->password = (char *) 0;
+		s->group = NULL;
 		s->away = (char *) 0;
 		s->version_string = (char *) 0;
 		s->server2_8 = 0;
@@ -112,9 +118,14 @@ void 	add_to_server_list (const char *server, int port, const char *password, co
 		s->reconnects = 0;
 		s->quit_message = NULL;
 		s->save_channels = -1;
+#ifdef HAVE_SSL
+		s->enable_ssl = FALSE;
+#endif
 
 		if (password && *password)
 			malloc_strcpy(&s->password, password);
+		if (group && *group)
+			malloc_strcpy(&s->group, group);
 		if (nick && *nick)
 			malloc_strcpy(&s->d_nickname, nick);
 		else if (!s->d_nickname)
@@ -173,6 +184,7 @@ static 	void 	remove_from_server_list (int i)
 	new_free(&s->name);
 	new_free(&s->itsname);
 	new_free(&s->password);
+	new_free(&s->group);
 	new_free(&s->away);
 	new_free(&s->version_string);
 	new_free(&s->nickname);
@@ -180,6 +192,18 @@ static 	void 	remove_from_server_list (int i)
 	new_free(&s->d_nickname);
 	new_free(&s->userhost);
 	new_free(&s->cookie);
+	new_free(&s->umodes);
+	new_free(&s->ison_queue);
+	new_free(&s->who_queue);
+	destroy_notify_list(i);
+
+#ifdef HAVE_SSL
+	if (server_list[i].enable_ssl == TRUE)
+	{
+		SSL_free((SSL *)&s->ssl_fd);
+		SSL_CTX_free((SSL_CTX *)&s->ctx);
+	}
+#endif
 
 	memmove(&server_list[i], &server_list[i + 1], 
 			(number_of_servers - i - 1) * sizeof(Server));
@@ -262,14 +286,16 @@ int	find_in_server_list (const char *server, int port)
  *
  *		refnum	 (where refnum is an integer for an existing server)
  * or
- *		hostname:port:password:nickname
+ *		hostname:port:password:nickname:group
  * or
- *		hostname port password nickname
+ *		hostname port password nickname group
  *
  * This extracts the salient information from the string and returns the
  * server_list index for that server.  If the information describes a server
  * that is not in the server_list, that information is *automatically added*
- * to the server_list and a new index is returned.
+ * to the server_list and a new index is returned.  Double-quoted words are
+ * supported as long as you use them reasonably.  Double-quoted words protect
+ * any colons on the inside from inadvertant mis-parsing.
  *
  * This function always succeeds.
  */
@@ -279,7 +305,8 @@ int 	find_server_refnum (char *server, char **rest)
 	int	port = irc_port;
 	char 	*cport = NULL, 
 		*password = NULL,
-		*nick = NULL;
+		*nick = NULL,
+		*group = NULL;
 
 	/*
 	 * First of all, check for an existing server refnum
@@ -291,16 +318,17 @@ int 	find_server_refnum (char *server, char **rest)
 	 * Next check to see if its a "server:port:password:nick"
 	 */
 	else if (strchr(server, ':'))
-		parse_server_info(server, &cport, &password, &nick);
+		parse_server_info(server, &cport, &password, &nick, &group);
 
 	/*
 	 * Next check to see if its "server port password nick"
 	 */
 	else if (rest && *rest)
 	{
-		cport = next_arg(*rest, rest);
-		password = next_arg(*rest, rest);
-		nick = next_arg(*rest, rest);
+		cport = new_next_arg(*rest, rest);
+		password = new_next_arg(*rest, rest);
+		nick = new_next_arg(*rest, rest);
+		group = new_next_arg(*rest, rest);
 	}
 
 	if (cport && *cport)
@@ -310,7 +338,7 @@ int 	find_server_refnum (char *server, char **rest)
 	 * Add to the server list (this will update the port
 	 * and password fields).
 	 */
-	add_to_server_list(server, port, password, nick, 1);
+	add_to_server_list(server, port, password, nick, group, 1);
 	return from_server;
 }
 
@@ -342,8 +370,11 @@ int	parse_server_index (const char *str)
  * string, so * upon return, name will only point the the name.  If portnum
  * or password are missing or empty,  their respective returned value will
  * point to null. 
+ *
+ * "*group" must be set to something (even if it is NULL) before calling this!
+ * Colons can be backslashed.
  */
-void	parse_server_info (char *name, char **port, char **password, char **nick)
+void	parse_server_info (char *name, char **port, char **password, char **nick, char **group)
 {
 	char *ptr;
 
@@ -351,29 +382,54 @@ void	parse_server_info (char *name, char **port, char **password, char **nick)
 
 	do
 	{
-		if (!(ptr = strchr(name, ':')))
+		ptr = name;
+		if (*ptr == '"')
+			name = new_next_arg(ptr, &ptr);
+		ptr = strchr(ptr, ':');
+		if (!ptr)
 			break;
 		*ptr++ = 0;
 		if (!*ptr)
 			break;
 		*port = ptr;
 
-		if (!(ptr = strchr(ptr, ':')))
+		if (*ptr == '"')
+			*port = new_next_arg(ptr, &ptr);
+		ptr = strchr(ptr, ':');
+		if (!ptr)
 			break;
 		*ptr++ = 0;
 		if (!*ptr)
 			break;
 		*password = ptr;
 
-		if (!(ptr = strchr(ptr, ':')))
+		if (*ptr == '"')
+			*password = new_next_arg(ptr, &ptr);
+		ptr = strchr(ptr, ':');
+		if (!ptr)
 			break;
 		*ptr++ = 0;
 		if (!*ptr)
 			break;
 		*nick = ptr;
 
+		if (*ptr == '"')
+			*nick = new_next_arg(ptr, &ptr);
+		ptr = strchr(ptr, ':');
+		if (!ptr)
+			break;
+		*ptr++ = 0;
+		if (!*ptr)
+			break;
+		*group = ptr;
+
 		/* Ignore any additional, future fields */
-		if ((ptr = strchr(ptr, ':')) != NULL)
+		if (*ptr == '"')
+			*group = new_next_arg(ptr, &ptr);
+		ptr = strchr(ptr, ':');
+		if (!ptr)
+			break;
+		else
 			*ptr++ = 0;
 	}
 	while (0);
@@ -394,10 +450,12 @@ void	parse_server_info (char *name, char **port, char **password, char **nick)
  *
  * servername::password 
  *
+ * servername::password:servergroup
+ *
  * Note also that this routine mucks around with the server string passed to it,
  * so make sure this is ok 
  */
-void	build_server_list (char *servers)
+void	build_server_list (char *servers, char *group)
 {
 	char	*host,
 		*rest,
@@ -412,17 +470,17 @@ void	build_server_list (char *servers)
 	while (servers)
 	{
 		if ((rest = strchr(servers, '\n')))
-			*rest++ = '\0';
+			*rest++ = 0;
 
 		while ((host = next_arg(servers, &servers)))
 		{
-			parse_server_info(host, &port, &password, &nick);
+			parse_server_info(host, &port, &password, &nick, &group);
                         if (port && *port && (port_num = my_atol(port)))
 				;
 			else
 				port_num = irc_port;
 
-			add_to_server_list(host, port_num, password, nick, 0);
+			add_to_server_list(host, port_num, password, nick, group, 0);
 		}
 		servers = rest;
 	}
@@ -438,6 +496,7 @@ int 	read_server_file (void)
 	char	file_path[MAXPATHLEN + 1];
 	char	buffer[BIG_BUFFER_SIZE + 1];
 	char	*expanded;
+	char	*defaultgroup = NULL;
 
 	if (getenv("IRC_SERVERS_FILE"))
 		strmcpy(file_path, getenv("IRC_SERVERS_FILE"), MAXPATHLEN);
@@ -455,19 +514,32 @@ int 	read_server_file (void)
 
 	if ((expanded = expand_twiddle(file_path)))
 	{
-		if ((fp = fopen(expanded, "r")))
+	    if ((fp = fopen(expanded, "r")))
+	    {
+		while (fgets(buffer, BIG_BUFFER_SIZE, fp))
 		{
-			while (fgets(buffer, BIG_BUFFER_SIZE, fp))
+			chop(buffer, 1);
+			if (*buffer == '#')
+				continue;
+			else if (*buffer == '[')
 			{
-				chop(buffer, 1);
-				build_server_list(buffer);
+			    char *p;
+			    if ((p = strrchr(buffer, ']')))
+				*p++ = 0;
+			    malloc_strcpy(&defaultgroup, buffer + 1);
 			}
-			fclose(fp);
-			new_free(&expanded);
-			return (0);
+			else if (*buffer == 0)
+				continue;
+			else
+				build_server_list(buffer, defaultgroup);
 		}
+		fclose(fp);
+		new_free(&defaultgroup);
+		return (0);
+	    }
 
-		new_free(&expanded);
+	    new_free(&expanded);
+	    new_free(&defaultgroup);
 	}
 	return 1;
 }
@@ -502,22 +574,25 @@ void 	display_server_list (void)
 	{
 		if (!server_list[i].nickname)
 		{
-			say("\t%d) %s %d", i,
+			say("\t%d) %s %d [%s]", i,
 				server_list[i].name,
-				server_list[i].port);
+				server_list[i].port,
+				get_server_group(i));
 		}
 		else
 		{
 			if (is_server_open(i))
-				say("\t%d) %s %d (%s)", i,
+				say("\t%d) %s %d (%s) [%s]", i,
 					server_list[i].name,
 					server_list[i].port,
-					server_list[i].nickname);
+					server_list[i].nickname,
+					get_server_group(i));
 			else
-				say("\t%d) %s %d (was %s)", i,
+				say("\t%d) %s %d (was %s) [%s]", i,
 					server_list[i].name,
 					server_list[i].port,
-					server_list[i].nickname);
+					server_list[i].nickname,
+					get_server_group(i));
 		}
 	}
 }
@@ -526,13 +601,14 @@ char *	create_server_list (void)
 {
 	int	i;
 	char	*buffer = NULL;
+	size_t	bufclue = 0;
 
 	for (i = 0; i < number_of_servers; i++)
 	{
 		if (server_list[i].des != -1)
 		{
 			if (server_list[i].itsname)
-				m_s3cat(&buffer, space, server_list[i].itsname);
+				m_sc3cat(&buffer, space, server_list[i].itsname, &bufclue);
 			else
 				yell("Warning: I don't have server #%d's real"
 					"name yet -- using the hostname you "
@@ -559,6 +635,9 @@ BUILT_IN_COMMAND(servercmd)
 {
 	char	*server = NULL;
 	int	i;
+#ifdef HAVE_SSL
+	int     ssl_connect = FALSE;
+#endif
 
 	if ((server = next_arg(args, &args)) == NULL)
 	{
@@ -566,6 +645,21 @@ BUILT_IN_COMMAND(servercmd)
 		return;
 	}
 
+#ifdef HAVE_SSL
+	if ((i = find_in_server_list(server, 0)) != -1)
+		server_list[i].enable_ssl=FALSE;
+	if (strlen(server) > 1 && !my_strnicmp(server, "-SSL", strlen(server)))
+	{
+		if (!(server = new_next_arg(args, &args)))
+		{
+			say("Not enough paramters - supply server name please");
+			return;
+		}
+		say("Trying to establish SSL connection with server: %s", server);
+		ssl_connect=TRUE;
+	}
+#endif
+
 	/*
 	 * Delete an existing server
 	 */
@@ -619,7 +713,14 @@ BUILT_IN_COMMAND(servercmd)
 
 		/* /SERVER +foo.bar.com is an alias for /window server */
 		if (*++server)
+		{
+#ifdef HAVE_SSL
+			i = find_server_refnum(server, &args);
+			if (ssl_connect == TRUE)
+				server_list[i].enable_ssl = TRUE;
+#endif
 			window_server(current_window, &server);
+		}
 
 		/* /SERVER + means go to the next server */
 		else
@@ -627,7 +728,7 @@ BUILT_IN_COMMAND(servercmd)
 			set_server_quit_message(from_server, 
 					"Changing servers");
 			server_reconnects_to(from_server, from_server + 1);
-			reconnect(from_server);
+			reconnect(from_server, 1);
 		}
 
 		window_check_servers();
@@ -652,7 +753,7 @@ BUILT_IN_COMMAND(servercmd)
 			set_server_quit_message(from_server, 
 					"Disconnected at user request");
 			server_reconnects_to(i, -1);
-			reconnect(i);
+			reconnect(i, 1);
 		}
 		else
 		{
@@ -664,7 +765,7 @@ BUILT_IN_COMMAND(servercmd)
 			else
 				server_reconnects_to(from_server, 
 							from_server - 1);
-			reconnect(from_server);
+			reconnect(from_server, 1);
 		}
 
 		window_check_servers();
@@ -681,8 +782,12 @@ BUILT_IN_COMMAND(servercmd)
 		if (i != j)
 		{
 			clear_reconnect_counts();
+#ifdef HAVE_SSL
+			if (ssl_connect == TRUE)
+				server_list[i].enable_ssl = TRUE;
+#endif
 			server_reconnects_to(j, i);
-			reconnect(j);
+			reconnect(j, 0);
 			window_check_servers();
 		}
 		else
@@ -712,12 +817,29 @@ void	do_server (fd_set *rd)
 
 		des = server_list[i].des;
 		if (des == -1 || !FD_ISSET(des, rd))
-			continue;
+		{
+#ifdef HAVE_SSL
+			if (server_list[i].enable_ssl == TRUE)
+			{
+				if (!SSL_pending(server_list[i].ssl_fd))
+					continue;
+			}
+			else
+#endif
+ 			continue;
+		}
 		FD_CLR(des, rd);	/* Make sure it never comes up again */
 
 		last_server = from_server = i;
-		switch ((junk = dgets(bufptr, des, 1)))
-		{
+#ifdef HAVE_SSL
+		if (server_list[i].enable_ssl == TRUE)
+			junk = SSL_dgets(bufptr, des, 1, IO_BUFFER_SIZE, server_list[i].ssl_fd);
+		else
+#endif
+		junk = dgets(bufptr, des, 1);
+
+		switch (junk)
+ 		{
 			case 0:		/* Sit on incomplete lines */
 				break;
 
@@ -733,7 +855,7 @@ void	do_server (fd_set *rd)
 					(dgets_errno == -1) ? 
 					     "Remote end closed connection" : 
 					     strerror(dgets_errno));
-				reconnect(i);
+				reconnect(i, 1);
 				i++;		/* NEVER DELETE THIS! */
 				break;
 			}
@@ -752,6 +874,8 @@ void	do_server (fd_set *rd)
 					yell("[%d] <- [%s]", 
 						server_list[i].des, buffer);
 
+				if (translation)
+					translate_from_server(buffer);
 				parsing_server_index = i;
 				parse_server(buffer);
 				parsing_server_index = -1;
@@ -767,59 +891,10 @@ void	do_server (fd_set *rd)
 		window_check_servers();
 }
 
-/*
- * flush_server: eats all output from server, until there is at least a
- * second delay between bits of servers crap... useful to abort a /links. 
- * If you do this, you may lose JOINs/PARTs/QUITs, etc which have a severe
- * impact on the consistency of the client's internal data.
- *
- * XXXX - this is utterly useless at best, and pointless at worst.
- */
-void 	flush_server (void)
-{
-	fd_set rd;
-	struct timeval timeout;
-	int	flushing = 1;
-	int	des;
-	char	buffer[IO_BUFFER_SIZE + 1];
-
-	if ((des = server_list[from_server].des) == -1)
-		return;
-	timeout.tv_usec = 0;
-	timeout.tv_sec = 1;
-	while (flushing)
-	{
-		FD_ZERO(&rd);
-		FD_SET(des, &rd);
-		switch (new_select(&rd, (fd_set *) 0, &timeout))
-		{
-			case -1:
-			case 0:
-				flushing = 0;
-				break;
-			default:
-			{
-				if (FD_ISSET(des, &rd))
-				{
-					FD_CLR(des, &rd);
-					if (dgets(buffer, des, 0) == 0)
-						flushing = 0;
-				}
-				break;
-			}
-		}
-	}
-
-	/* make sure we've read a full line from server */
-	FD_ZERO(&rd);
-	FD_SET(des, &rd);
-	if (new_select(&rd, (fd_set *) 0, &timeout) > 0)
-		dgets(buffer, des, 1);
-}
-
 
 /* SERVER OUTPUT STUFF */
 static void 	vsend_to_server (const char *format, va_list args);
+void		send_to_server_raw (size_t len, const char *buffer);
 
 void	send_to_aserver (int refnum, const char *format, ...)
 {
@@ -859,6 +934,9 @@ static void 	vsend_to_server (const char *format, va_list args)
 		/* Keep the results short, and within reason. */
 		len = vsnprintf(buffer, BIG_BUFFER_SIZE, format, args);
 
+		if (translation)
+			translate_to_server(buffer);
+
 		if (outbound_line_mangler)
 		{
 			if (mangle_line(buffer, outbound_line_mangler, size) 
@@ -875,16 +953,7 @@ static void 	vsend_to_server (const char *format, va_list args)
 		if (do_hook(SEND_TO_SERVER_LIST, "%d %d %s", 
 				server, des, buffer))
 		{
-		    if (write(des, buffer, strlen(buffer)) == -1 && 
-			(!get_int_var(NO_FAIL_DISCONNECT_VAR)))
-		    {
-			if (server_list[server].connected)
-			{
-			    server_is_connected(des, 0);
-			    say("Write to server failed.  Closing connection.");
-			    reconnect(server);
-			}
-		    }
+			send_to_server_raw (strlen(buffer), buffer);
 		}
 	}
 	else if (from_server == -1)
@@ -895,6 +964,50 @@ static void 	vsend_to_server (const char *format, va_list args)
         }
 }
 
+void	send_to_server_raw (size_t len, const char *buffer)
+{
+	int des, server;
+	int err = 0;
+
+	if ((server = from_server) == -1)
+		server = primary_server;
+
+	if (server != -1 && (des = server_list[server].des) != -1 && buffer)
+	{
+#ifdef HAVE_SSL
+		if (server_list[server].enable_ssl == TRUE)
+		{
+			if (server_list[server].ssl_fd == 0)
+			{
+				say("SSL write error - ssl socket=0");
+				return;
+			}
+			err = SSL_write(server_list[server].ssl_fd, buffer, strlen(buffer));
+			BIO_flush(SSL_get_wbio(server_list[server].ssl_fd));
+		}
+		else
+#endif 
+		err = write(des, buffer, strlen(buffer));
+
+		if (err == -1 &&
+			(!get_int_var(NO_FAIL_DISCONNECT_VAR)))
+		{
+			if (server_list[server].connected)
+			{
+				/* server_list[server].save_channels == 1; */
+				/* close_server(server, strerror(errno));  */
+				server_is_connected(des, 0);
+				say("Write to server failed.  Closing connection.");
+#ifdef HAVE_SSL
+				if (server_list[server].enable_ssl == TRUE)
+					SSL_shutdown(server_list[server].ssl_fd);
+#endif
+				reconnect(server, 1);
+			}
+		}
+	}
+}
+
 void	clear_sent_to_server (int servnum)
 {
 	server_list[servnum].sent = 0;
@@ -905,6 +1018,14 @@ int	sent_to_server (int servnum)
 	return server_list[servnum].sent;
 }
 
+void	flush_server (int servnum)
+{
+	if (!is_server_connected(servnum))
+		return;
+	set_server_redirect(servnum, "0");
+	send_to_aserver(servnum, "%s", "***0");
+}
+
 
 /* CONNECTION/RECONNECTION STRATEGIES */
 /*
@@ -972,6 +1093,9 @@ static int 	connect_to_server (int new_server)
 
 		say("Unable to connect to port %d of server %s: %s", 
 				s->port, s->name, my_strerror(errno));
+#ifdef HAVE_SSL
+		s->enable_ssl = FALSE; /* Would cause client to crash, if not wiped out */
+#endif
 		return -1;		/* Connect failed */
 	}
 
@@ -993,7 +1117,7 @@ static int 	connect_to_server (int new_server)
 	 * Initialize all of the server_list data items
 	 * XXX - Calling add_to_server_list is a hack.
 	 */
-	add_to_server_list(s->name, s->port, NULL, NULL, 1);
+	add_to_server_list(s->name, s->port, NULL, NULL, NULL, 1);
 	s->des = des;
 	s->oper = 0;
 	if (!s->d_nickname)
@@ -1103,7 +1227,10 @@ int 	connect_to_new_server (int new_server, int old_server, int new_conn)
 		 * establishing a brand new connection, we 
 		 */
 		if (x == -2)
+		{
+			destroy_waiting_channels(old_server);
 			destroy_server_channels(old_server);
+		}
 	
 		/*
 		 * If we are /server'ing to a server that was not already 
@@ -1168,8 +1295,14 @@ int 	connect_to_new_server (int new_server, int old_server, int new_conn)
  * a normal reconnect act.  Setting the reconnecting server to another refnum
  * causes the server connection to be migrated to that refnum.  The original
  * connection is closed only if the new refnum is successfully opened.
+ *
+ * The "samegroup" argument is new and i think every call to reconnect
+ * uses the value '1' which means 'only try to connect to the same group
+ * as whatever we're reconnecting to'; the code for value 0 may not be
+ * tested very well, and i can't think of any uses for it, but eh, you 
+ * never know what the future will hold.
  */
-int	reconnect (int oldserv)
+int	reconnect (int oldserv, int samegroup)
 {
 	int	newserv;
 	int	i, j;
@@ -1221,6 +1354,10 @@ int	reconnect (int oldserv)
 	for (i = 0; i < number_of_servers; i++)
 	{
 		j = (i + newserv) % number_of_servers;
+		if (samegroup && oldserv >= 0 &&
+			my_stricmp(get_server_group(oldserv), 
+				   get_server_group(j)))
+			continue;
 		if (newserv != oldserv && j == oldserv)
 			continue;
 		if ((server_list[j].reconnects++) > max_reconnects) {
@@ -1265,7 +1402,10 @@ int	reconnect (int oldserv)
 	 * then we need to throw away it's channels.
 	 */
 	if (!is_server_connected(oldserv))
+	{
+		destroy_waiting_channels(oldserv);
 		destroy_server_channels(oldserv);
+	}
 
 	/*
 	 * Our prior state was unconnected.  Tell the user
@@ -1288,7 +1428,7 @@ int 	close_all_servers (const char *message)
 	{
 		set_server_quit_message(i, message);
 		server_reconnects_to(i, -1);
-		reconnect(i);
+		reconnect(i, 0);
 	}
 
 	return 0;
@@ -1311,9 +1451,6 @@ void	close_server (int old, const char *message)
 		return;
 	}
 
-	if (server_list[old].des == -1)
-		return;			/* Can't close server that isn't open */
-
 	was_connected = server_list[old].connected;
 
 	clean_server_queues(old);
@@ -1323,7 +1460,10 @@ void	close_server (int old, const char *message)
 	if (server_list[old].save_channels == 1)
 		save_channels(old);
 	else if (server_list[old].save_channels == 0)
+	{
+		destroy_waiting_channels(old);
 		destroy_server_channels(old);
+	}
 	else
 		panic("Somebody forgot to set "
 			"server_list[%d].save_channels!", old);
@@ -1352,6 +1492,13 @@ void	close_server (int old, const char *message)
 		     */
 		    if (was_connected)
 			    send_to_aserver(old, "QUIT :%s\n", message);
+#ifdef HAVE_SSL
+		    if (server_list[old].enable_ssl==TRUE)
+		    {
+			    say("Closing SSL connection");
+			    SSL_shutdown(server_list[old].ssl_fd);
+		    }
+#endif
 		    do_hook(SERVER_LOST_LIST, "%d %s %s", 
 				old, server_list[old].name, message);
 		}
@@ -1598,10 +1745,53 @@ const char *	get_server_version_string (int servnum)
 	 return server_list[servnum].version_string;
 }
 
+#ifdef HAVE_SSL
+
+void	set_server_enable_ssl (int gsn_index, int value)
+{
+	server_list[gsn_index].enable_ssl = value;
+}
+
+/* get_server_isssl: returns 1 if the server is using SSL connection */
+int	get_server_isssl (int gsn_index)
+{
+	if (gsn_index == -1)
+		gsn_index = primary_server;
+	else if (gsn_index == -1 || gsn_index >= number_of_servers)
+		return 0;
+
+	return ((server_list[gsn_index].enable_ssl == TRUE) ? (1) : (0));
+}
+
+const char	*get_server_cipher (int gsn_index)
+{
+	if (gsn_index == -1)
+		gsn_index = primary_server;
+	else if (gsn_index == -1 || gsn_index >= number_of_servers
+		|| server_list[gsn_index].enable_ssl == FALSE)
+		return empty_string;
+
+	return (SSL_get_cipher(server_list[gsn_index].ssl_fd));
+}
+
+#else
+
+/* get_server_isssl: returns 1 if the server is using SSL connection */
+int	get_server_isssl (int gsn_index)
+{
+	return 0;
+}
+#endif
 
 /* CONNECTION/REGISTRATION STATUS */
 void	register_server (int ssn_index, const char *nickname)
 {
+#ifdef HAVE_SSL
+	int		alg;
+	int		sign_alg;
+	X509		*server_cert;
+	EVP_PKEY	*server_pkey;
+#endif;
 	if (server_list[ssn_index].registration_pending)
 		return;		/* Whatever */
 
@@ -1609,6 +1799,60 @@ void	register_server (int ssn_index, const char *nickname)
 		return;		/* Whatever */
 
 	server_list[ssn_index].registration_pending = 1;
+#ifdef HAVE_SSL
+	if (server_list[ssn_index].enable_ssl == TRUE)
+	{
+		say("SSL negotiation in progress...");
+/* Old SSL connection routines.
+		SSLeay_add_ssl_algorithms();
+		SSL_load_error_strings();
+		server_list[ssn_index].ctx = SSL_CTX_new(SSLv3_client_method());
+		server_list[ssn_index].ssl_fd = SSL_new(server_list[ssn_index].ctx);
+		SSL_set_fd(server_list[ssn_index].ssl_fd, server_list[ssn_index].des);
+		SSL_connect(server_list[ssn_index].ssl_fd);
+*/		
+		/* Set up SSL connection */
+		server_list[ssn_index].ctx = SSL_CTX_init(0);
+		server_list[ssn_index].ssl_fd = SSL_FD_init(server_list[ssn_index].ctx,
+			server_list[ssn_index].des);
+
+		if (x_debug & DEBUG_SSL)
+			say("SSL negotiation using %s",
+				get_server_cipher(ssn_index));
+		say("SSL negotiation on port %d of server %s complete",
+			server_list[ssn_index].port, get_server_name(ssn_index));
+		server_cert = SSL_get_peer_certificate(server_list[ssn_index].ssl_fd);
+		if (!(server_cert == NULL))
+		{
+			server_pkey = X509_get_pubkey(server_cert);
+			/* urlencoded to avoid problems with spaces */
+			do_hook(SSL_SERVER_CERT_LIST, "%s %s %s",
+				server_list[ssn_index].name,
+				urlencode(X509_NAME_oneline(X509_get_subject_name(server_cert),0,0)),
+				urlencode(X509_NAME_oneline(X509_get_issuer_name(server_cert),0,0)));
+
+			/* We should check certificate date validity here. */
+
+			if (x_debug & DEBUG_SSL)
+			{
+				alg = OBJ_obj2nid(server_cert->cert_info->key->algor->algorithm);
+				sign_alg = OBJ_obj2nid(server_cert->sig_alg->algorithm);		
+				say("Public key algorithm: %s (%d bits)  Sign algorithm: %s",
+					(alg == NID_undef) ? "UNKNOWN" : OBJ_nid2ln(alg), EVP_PKEY_bits(server_pkey),
+					(sign_alg == NID_undef) ? "UNKNOWN" : OBJ_nid2ln(sign_alg));
+			}
+			EVP_PKEY_free(server_pkey);
+		}
+		else
+		{
+			/* No server certificate found */
+			do_hook(SSL_SERVER_CERT_LIST, "%s %s %s",
+				server_list[ssn_index].name,
+				empty_string, empty_string);
+		}
+		X509_free(server_cert);
+	}
+#endif
 	if (server_list[ssn_index].password)
 		send_to_aserver(ssn_index, "PASS %s", 
 			server_list[ssn_index].password);
@@ -1635,7 +1879,7 @@ void 	password_sendline (char *data, char *line)
 		set_server_password(new_server, line);
 		change_window_server(new_server, new_server);
 		server_reconnects_to(new_server, new_server);
-		reconnect(new_server);
+		reconnect(new_server, 1);
 	}
 }
 
@@ -1767,7 +2011,7 @@ int 	auto_reconnect_callback (void *d)
 	new_free((char **)&d);
 
 	server_reconnects_to(servref, servref);
-	reconnect(servref);
+	reconnect(servref, 1);
 	return 0;
 }
 
@@ -2158,6 +2402,8 @@ int	check_server_redirect (const char *who)
 	    !strcmp(who + 3, server_list[from_server].redirect))
 	{
 		set_server_redirect(from_server, NULL);
+		if (!strcmp(who + 3, "0"))
+			say("Server flush done.");
 		return 1;
 	}
 
@@ -2178,13 +2424,15 @@ void 	save_servers (FILE *fp)
 	for (i = 0; i < number_of_servers; i++)
 	{
 		/* SERVER -ADD server:port:password:nick */
-		fprintf(fp, "SERVER -ADD %s:%d:%s:%s\n",
+		fprintf(fp, "SERVER -ADD %s:%d:%s:%s:%s\n",
 			server_list[i].name,
 			server_list[i].port,
 			server_list[i].password ? 
 				server_list[i].password : empty_string,
 			server_list[i].nickname ?
-				server_list[i].nickname : empty_string);
+				server_list[i].nickname : empty_string,
+			server_list[i].group ?
+				server_list[i].group : empty_string);
 	}
 }
 
@@ -2195,4 +2443,17 @@ void	clear_reconnect_counts (void)
 	for (i = 0; i < number_of_servers; i++)
 		server_list[i].reconnects = 0;
 }
+/* This didn't belong in the middle of the redirect stuff. */
+const char *get_server_group (int refnum)
+{
+	if (refnum == -1 && from_server != -1)
+		refnum = from_server;
 
+	if (refnum >= number_of_servers || refnum < 0)
+		return 0;
+
+	if (server_list[refnum].group == NULL)
+		return "<default>";
+
+	return server_list[refnum].group;
+}
