@@ -9,7 +9,7 @@
  */
 
 #if 0
-static	char	rcsid[] = "$Id: vars.c,v 1.13 1994/03/20 14:10:29 mrgreen Exp $";
+static	char	rcsid[] = "$Id: vars.c,v 1.11 2002/02/17 22:40:29 crazyed Exp $";
 #endif
 
 #include "irc.h"
@@ -30,6 +30,7 @@ static	char	rcsid[] = "$Id: vars.c,v 1.13 1994/03/20 14:10:29 mrgreen Exp $";
 #include "stack.h"
 #include "dcc.h"
 #include "keys.h"
+#include "translat.h"
 
 /* IrcVariable: structure for each variable in the variable table */
 typedef struct
@@ -82,12 +83,14 @@ static	void	set_hold_interval	(int value);
  */
 static	IrcVariable irc_variable[] =
 {
+	{ "ALLOW_C1_CHARS",		BOOL_TYPE_VAR,	DEFAULT_ALLOW_C1_CHARS, NULL, NULL, 0, 0 },
 	{ "ALT_CHARSET",		BOOL_TYPE_VAR,	DEFAULT_ALT_CHARSET, NULL, NULL, 0, 0 },
 	{ "ALWAYS_SPLIT_BIGGEST",	BOOL_TYPE_VAR,	DEFAULT_ALWAYS_SPLIT_BIGGEST, NULL, NULL, 0, 0 },
 	{ "AUTO_NEW_NICK",		BOOL_TYPE_VAR,	DEFAULT_AUTO_NEW_NICK, NULL, NULL, 0, 0 },
         { "AUTO_RECONNECT",             BOOL_TYPE_VAR,  DEFAULT_AUTO_RECONNECT, NULL, NULL, 0, 0 },
 	{ "AUTO_RECONNECT_DELAY",	INT_TYPE_VAR,	DEFAULT_AUTO_RECONNECT_DELAY, NULL, NULL, 0, 0 },
         { "AUTO_REJOIN",                BOOL_TYPE_VAR,  DEFAULT_AUTO_REJOIN, NULL, NULL, 0, 0 },
+	{ "AUTO_REJOIN_CONNECT",	BOOL_TYPE_VAR,	DEFAULT_AUTO_REJOIN_CONNECT, NULL, NULL, 0, 0 },
 	{ "AUTO_REJOIN_DELAY",		INT_TYPE_VAR,	DEFAULT_AUTO_REJOIN_DELAY, NULL, NULL, 0, 0 },
 	{ "AUTO_UNMARK_AWAY",		BOOL_TYPE_VAR,	DEFAULT_AUTO_UNMARK_AWAY, NULL, NULL, 0, 0 },
 	{ "AUTO_WHOWAS",		BOOL_TYPE_VAR,	DEFAULT_AUTO_WHOWAS, NULL, NULL, 0, 0 },
@@ -165,7 +168,6 @@ static	IrcVariable irc_variable[] =
         { "MODE_STRIPPER",              BOOL_TYPE_VAR,  DEFAULT_MODE_STRIPPER, NULL, NULL, 0, 0 },
 	{ "ND_SPACE_MAX",		INT_TYPE_VAR,	DEFAULT_ND_SPACE_MAX, NULL, NULL, 0, 0 },
 	{ "NEW_SERVER_LASTLOG_LEVEL",	STR_TYPE_VAR,	0, NULL, set_new_server_lastlog_level, 0, 0 },
-	{ "NOTIFY_HANDLER",		STR_TYPE_VAR, 	0, NULL, set_notify_handler, 0, 0 },
 	{ "NOTIFY_INTERVAL",		INT_TYPE_VAR,	DEFAULT_NOTIFY_INTERVAL, NULL, NULL, 0, 0 },
 	{ "NOTIFY_LEVEL",		STR_TYPE_VAR,	0, NULL, set_notify_level, 0, 0 },
 	{ "NOTIFY_ON_TERMINATION",	BOOL_TYPE_VAR,	DEFAULT_NOTIFY_ON_TERMINATION, NULL, NULL, 0, 0 },
@@ -195,6 +197,9 @@ static	IrcVariable irc_variable[] =
 	{ "SHOW_NUMERICS",		BOOL_TYPE_VAR,	DEFAULT_SHOW_NUMERICS, NULL, NULL, 0, 0 },
 	{ "SHOW_STATUS_ALL",		BOOL_TYPE_VAR,	DEFAULT_SHOW_STATUS_ALL, NULL, update_all_status, 0, 0 },
 	{ "SHOW_WHO_HOPCOUNT", 		BOOL_TYPE_VAR,	DEFAULT_SHOW_WHO_HOPCOUNT, NULL, NULL, 0, 0 },
+	{ "SSL_CERTFILE",		STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
+	{ "SSL_KEYFILE",		STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
+	{ "SSL_PATH",			STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
 	{ "STATUS_AWAY",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_CHANNEL",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_CHANOP",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
@@ -204,6 +209,7 @@ static	IrcVariable irc_variable[] =
 	{ "STATUS_FORMAT",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_FORMAT1",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_FORMAT2",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_HALFOP",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_HOLD",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_HOLD_LINES",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_INSERT",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
@@ -217,6 +223,8 @@ static	IrcVariable irc_variable[] =
 	{ "STATUS_QUERY",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_SCROLLBACK",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_SERVER",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_SSL_OFF",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_SSL_ON",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_TRUNCATE_RHS",	BOOL_TYPE_VAR,	DEFAULT_STATUS_TRUNCATE_RHS, NULL, build_status, 0, 0 },
 	{ "STATUS_UMODE",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_USER",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
@@ -267,10 +275,13 @@ static	IrcVariable irc_variable[] =
 	{ "TAB",			BOOL_TYPE_VAR,	DEFAULT_TAB, NULL, NULL, 0, 0 },
 	{ "TAB_MAX",			INT_TYPE_VAR,	DEFAULT_TAB_MAX, NULL, NULL, 0, 0 },
 	{ "TERM_DOES_BRIGHT_BLINK",	BOOL_TYPE_VAR,	DEFAULT_TERM_DOES_BRIGHT_BLINK, NULL, NULL, 0, 0 },
+	{ "TRANSLATION",		STR_TYPE_VAR,	0, NULL, set_translation, 0, 0 },
+	{ "TRANSLATION_PATH",		STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
 	{ "UNDERLINE_VIDEO",		BOOL_TYPE_VAR,	DEFAULT_UNDERLINE_VIDEO, NULL, NULL, 0, 0 },
 	{ "USER_INFORMATION", 		STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
 	{ "VERBOSE_CTCP",		BOOL_TYPE_VAR,	DEFAULT_VERBOSE_CTCP, NULL, NULL, 0, 0 },
 	{ "WORD_BREAK",			STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
+	{ "WSERV_PATH",			STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
 	{ "XTERM",			STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
 	{ "XTERM_OPTIONS", 		STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
 	{ (char *) 0, 0, 0, 0, 0, 0, 0 }
@@ -306,17 +317,22 @@ void 	init_variables (void)
 	set_string_var(MANGLE_OUTBOUND_VAR, NULL);
 	set_string_var(NEW_SERVER_LASTLOG_LEVEL_VAR, 
 			DEFAULT_NEW_SERVER_LASTLOG_LEVEL);
-	set_string_var(NOTIFY_HANDLER_VAR, DEFAULT_NOTIFY_HANDLER);
 	set_string_var(NOTIFY_LEVEL_VAR, DEFAULT_NOTIFY_LEVEL);
 	set_string_var(OUTPUT_REWRITE_VAR, NULL);
 	set_string_var(QUIT_MESSAGE_VAR, DEFAULT_QUIT_MESSAGE);
 	set_string_var(REALNAME_VAR, realname);
+	set_string_var(SSL_CERTFILE_VAR, NULL);
+	set_string_var(SSL_KEYFILE_VAR, NULL);
+	set_string_var(SSL_PATH_VAR, NULL);
 	set_string_var(STATUS_FORMAT_VAR, DEFAULT_STATUS_FORMAT);
 	set_string_var(STATUS_FORMAT1_VAR, DEFAULT_STATUS_FORMAT1);
 	set_string_var(STATUS_FORMAT2_VAR, DEFAULT_STATUS_FORMAT2);
 	set_string_var(STATUS_AWAY_VAR, DEFAULT_STATUS_AWAY);
 	set_string_var(STATUS_CHANNEL_VAR, DEFAULT_STATUS_CHANNEL);
 	set_string_var(STATUS_CHANOP_VAR, DEFAULT_STATUS_CHANOP);
+	set_string_var(STATUS_HALFOP_VAR, DEFAULT_STATUS_HALFOP);
+	set_string_var(STATUS_SSL_ON_VAR, DEFAULT_STATUS_SSL_ON);
+	set_string_var(STATUS_SSL_OFF_VAR, DEFAULT_STATUS_SSL_OFF);
 	set_string_var(STATUS_CLOCK_VAR, DEFAULT_STATUS_CLOCK);
 	set_string_var(STATUS_CPU_SAVER_VAR, DEFAULT_STATUS_CPU_SAVER);
 	set_string_var(STATUS_HOLD_VAR, DEFAULT_STATUS_HOLD);
@@ -373,12 +389,14 @@ void 	init_variables (void)
 	set_string_var(STATUS_USER39_VAR, DEFAULT_STATUS_USER39);
 	set_string_var(STATUS_VOICE_VAR, DEFAULT_STATUS_VOICE);
 	set_string_var(STATUS_WINDOW_VAR, DEFAULT_STATUS_WINDOW);
+	set_string_var(TRANSLATION_VAR, NULL);
 	set_string_var(USERINFO_VAR, DEFAULT_USERINFO);
 	set_string_var(XTERM_VAR, DEFAULT_XTERM);
 	set_string_var(XTERM_OPTIONS_VAR, DEFAULT_XTERM_OPTIONS);
 	set_string_var(STATUS_NOTIFY_VAR, DEFAULT_STATUS_NOTIFY);
 	set_string_var(CLIENTINFO_VAR, IRCII_COMMENT);
 	set_string_var(WORD_BREAK_VAR, DEFAULT_WORD_BREAK);
+	set_string_var(WSERV_PATH_VAR, WSERV_PATH);
 
 	/*
 	 * Construct the default help path
@@ -802,9 +820,10 @@ char	*get_all_sets (void)
 {
 	int	i;
 	char	*ret = NULL;
+	size_t	rclue = 0;
 
 	for (i = 0; irc_variable[i].name; i++)
-		m_s3cat(&ret, space, irc_variable[i].name);
+		m_sc3cat(&ret, space, irc_variable[i].name, &rclue);
 	return ret;
 }
 
@@ -813,13 +832,14 @@ char	*get_set (const char *str)
 {
 	int	i;
 	char	*ret = NULL;
+	size_t	rclue = 0;
 
 	if (!str || !*str)
 		return get_all_sets();
 
 	for (i = 0; irc_variable[i].name; i++)
 		if (wild_match(str, irc_variable[i].name))
-			m_s3cat(&ret, space, irc_variable[i].name);
+			m_sc3cat(&ret, space, irc_variable[i].name, &rclue);
 
 	return ret ? ret : m_strdup(empty_string);
 }
@@ -1060,7 +1080,7 @@ void do_stack_set(int type, char *args)
 			aptr->set = (IrcVariable *) new_malloc(sizeof(IrcVariable));
 			memcpy(aptr->set, &irc_variable[var_index], sizeof(IrcVariable));
 			aptr->name = m_strdup(irc_variable[var_index].name);
-			aptr->set->string = m_strdup(irc_variable[var_index].string);
+			aptr->set->string = (irc_variable[var_index].string) ? m_strdup(irc_variable[var_index].string) : NULL;
 			aptr->var_index = var_index;
 		}
 		else
